const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// Create Main Model

const Attachment = db.attachments

// Create
const addAttachment = async (req, res) => {
 try {
  let info = {
   id: uuidv4(),
   name: req.body.name,
   filename: req.body.filename,
   length: req.body.length,
   mimeType: req.body.mimeType,
   directory: req.body.directory,
   subdirectory: req.body.subdirectory,
   attachmentCategory: req.body.attachmentCategory,
   attachmentType: req.body.attachmentType,
   disabled: req.body.disabled
  }
 
  const attachment = await Attachment.create(info)
  return res.status(200).send(attachment)

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All
const getAllAttachments = async (req, res) => {
 try {
  let attachments = await Attachment.findAll({
   // attributes: [] // get specific column form database
   include: DataOptions
  })
  res.send(attachments)
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All Enabled
const getAttachments = async (req, res) => {
 try {
  let attachments = await Attachment.findAll({
   where: {
    [Op.or]: [
      { disabled: null },
      { disabled: false }
    ],
    [Op.and]: [
      { deletedAt: null }
    ]
   },
   include: DataOptions
  })
  res.send(attachments)
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get Filtered (Admin)
const getFilteredAttachments = async (req, res) => {
 try {
  let filter = {}
  for (let prop in req.body) {
    if (req.body[prop]) {
      filter[prop] = req.body[prop]
    }
  }

 let attachments = await Attachment.findAll({
  where: { ...filter },
  include: DataOptions
 })

 res.send(attachments)

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get One
const getOneAttachment = async (req, res) => {
 try {
  let attachmentId = req.params.id

  if (!attachmentId) return res.status(400).send({ statusCode: 400, message: 'Invalid Attachment ID' })

  let attachment = await Attachment.findOne({
   where: {id: attachmentId},
   include: DataOptions
  })
  return res.status(200).send(attachment)
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Update One
const updateAttachment = async (req, res) => {
 try {
  let attachmentId = req.params.id

  if (!attachmentId) return res.status(400).send({ statusCode: 400, message: 'Invalid Attachment ID' })

  const attachment = await Attachment.update(req.body, { where: { id: attachmentId } })
  return res.status(200).send(attachment)
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Delete One
const deleteAttachment = async (req, res) => {
 try {
  let attachmentId = req.params.id

  if (!attachmentId) return res.status(400).send({ statusCode: 400, message: 'Invalid Attachment ID' })

  await Attachment.destroy({
   where: {id: attachmentId}
  })
  return res.status(200).send(attachment)
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

module.exports = {
 addAttachment,
 getAllAttachments,
 getAttachments,
 getFilteredAttachments,
 getOneAttachment,
 updateAttachment,
 deleteAttachment
}