const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const moment = require('moment')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// Create Main Model

const Banner = db.banners

// Create
const add = async (req, res) => {
 try {

  let info = {
    id: uuidv4(),
    title: req.body.title,
    sequence: req.body.sequence,
    description: req.body.description,
    fromDate: req.body.fromDate,
    toDate: req.body.toDate,
    disabled: req.body.disabled,
    imageOnly: req.body.imageOnly,
    btnText: req.body.btnText,
    btnLink: req.body.btnLink,
    btnType: req.body.btnType ? req.body.btnType.value : null
   }
 
  await Banner.create(info)
    .then((banner) => {
      return res.status(200).send(banner)
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Create and Upload
const addAndUpload = async (req, res) => {
  try {
   const filePath = req.body.imagePath

   let info = {
     id: uuidv4(),
     title: req.body.title,
     sequence: req.body.sequence,
     description: req.body.description,
     fromDate: req.body.fromDate,
     toDate: req.body.toDate,
     attachments: filePath,
     disabled: req.body.disabled,
     imageOnly: req.body.imageOnly,
     btnText: req.body.btnText,
     btnLink: req.body.btnLink,
     btnType: req.body.btnType ? req.body.btnType.value : null
    }
  
   await Banner.create(info)
     .then((banner) => {
       return res.status(200).send(banner)
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
       })
     })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
}

const addAndUploadUpdate = async (req, res) => {
  try {
   const filePath = req.body.imagePath

   let info = {
     title: req.body.title,
     sequence: req.body.sequence,
     description: req.body.description,
     fromDate: req.body.fromDate,
     toDate: req.body.toDate,
     attachments: filePath,
     disabled: req.body.disabled,
     imageOnly: req.body.imageOnly,
     btnText: req.body.btnText,
     btnLink: req.body.btnLink,
     btnType: req.body.btnType ? req.body.btnType.value : null
    }
  
   await Banner.update(info, { where: { id: req.body.id } })
     .then(() => {
       Banner.findOne({ where: { id: req.body.id } })
        .then((banner) => {
          return res.status(200).send(banner)
        })
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
       })
     })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
}

// Get All
const getAll = async (req, res) => {
 try {
  let str = req.body && req.body.str
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0

  let filter = str ? { title: { [Op.substring] : '%' + str + '%' } } : ''

  await Banner.findAll({
    where: { ...filter },
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get All Enabled
const get = async (req, res) => {
 try {
  await Banner.findAll({
   where: {
    [Op.or]: [
      { disabled: null },
      { disabled: false }
    ],
    [Op.and]: [
      { deletedAt: null }
    ]
   }
  }).then((dataOptions) => {
    return res.status(200).send(dataOptions)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get Filtered (Admin)
const getFiltered = async (req, res) => {
 try {
  let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
  let filter = {}
 
   if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
      }
    }
  }
  
  await Banner.findAll({
    where: { ...filter },
    offset: page,
    limit: limit,
    order: [[sortBy, order]]
  }).then((response) => {
    return returnWithCount(res, filter, response, limit, true)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })

 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Get Filtered (Client)
const getFilteredClient = async (req, res) => {
  try {
   let sortBy = req.body.sortBy || 'brgyKey'
   let order = req.body.order || 'ASC'
   let limit = req.body.size ? req.body.size : 20
   let pageCount = req.body.page ? req.body.page : 1
   let limitCount = 0 + (pageCount - 1) * limit
   let page = limitCount > 0 ? limitCount : 0
   let filter = {}
  
    if (req.body.filter) {
     for (let prop in req.body.filter) {
       if (req.body.filter[prop]) {
         filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
       }
     }
   }
   
   await Banner.findAll({
     where: { ...filter,
        [Op.or]: [
          { disabled: null },
          { disabled: false }
        ],
        [Op.and]: [
          { fromDate: { [Op.lte]: moment().toDate() }},
          { toDate: { [Op.gte]: moment().toDate() }},
          { deletedAt: null }
        ]
      },
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
    return returnWithCount(res, filter, response, limit, false)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
     })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
}

// Get One
const getOne = async (req, res) => {
 try {
  let bannerId = req.params.id

  if (!bannerId) return res.status(400).send({ statusCode: 400, message: 'Invalid Banner ID' })

  await Banner.findOne({
   where: { id: bannerId },
  }).then((banner) => {
    return res.status(200).send(banner)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Update One
const update = async (req, res) => {
 try {
  let bannerId = req.params.id

  if (!bannerId) return res.status(400).send({ statusCode: 400, message: 'Invalid Banner ID' })

  let info = {
    title: req.body.title,
    sequence: req.body.sequence,
    description: req.body.description,
    fromDate: req.body.fromDate,
    toDate: req.body.toDate,
    disabled: req.body.disabled,
    imageOnly: req.body.imageOnly,
    btnText: req.body.btnText,
    btnLink: req.body.btnLink,
    btnType: req.body.btnType ? req.body.btnType.value : null
   }

  await Banner.update(info, { where: { id: bannerId } })
  .then((banner) => {
    return res.status(200).send(banner)
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Delete One
const remove = async (req, res) => {
 try {
  let bannerId = req.params.id

  if (!bannerId) return res.status(400).send({ statusCode: 400, message: 'Invalid Banner ID' })

  await Banner.destroy({
   where: {id: bannerId}
  }).then(() => {
    return res.status(200).send()
  }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
     })
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Return with count
const returnWithCount = async (res, filter, response, limit, isAdmin) => {
  if (isAdmin) {
    Banner.count({
      where: { ...filter}
    }).then((totalCount) => {
      let result = {
        content: response,
        totalElements: totalCount,
        totalPages: Math.ceil(totalCount/limit),
      }
      return res.status(200).send(result)
    })
  } else {
    Banner.count({
      where: { ...filter,
         [Op.or]: [
           { disabled: null },
           { disabled: false }
         ],
         [Op.and]: [
           { deletedAt: null }
         ]
       }}).then((totalCount) => {
          let result = {
            content: response,
            totalElements: totalCount,
            totalPages: Math.ceil(totalCount/limit),
          }
          return res.status(200).send(result)
      })
  }
  
}

module.exports = {
  add,
  addAndUpload,
  addAndUploadUpdate,
  getAll,
  get,
  getFiltered,
  getFilteredClient,
  getOne,
  update,
  remove,
  returnWithCount
}