const { verify } = require('jsonwebtoken')
const loginController = require('./login.controller')
const roleController = require('./role.controller')
const complaintController = require('./complaint.controller')
const countryController = require('./country.controller')
const regionController = require('./region.controller')
const cityController = require('./city.controller')
const barangayController = require('./barangay.controller')
const dataOptionController = require('./dataOption.controller')
const dataOptionTypeController = require('./dataOptionType.controller')
const userController = require('./user.controller')
const userDetailController = require('./userDetail.controller')
const bannerController = require('./banner.controller')
const announcementController = require('./announcement.controller')
const newsController = require('./news.controller')
const teamController = require('./team.controller')
const emergencyController = require('./emergency.controller')
const carStickerController = require('./carSticker.controller')
const mailController = require('./mail.controller')
const { reEvaluateRefreshToken } = require('../services/authenticate.service')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

const fePost = async (req, res) => {
 try {
  const type = req.params.type
  switch (type) {
   case 'role':
    return roleController.add(req, res)
   case 'complaint':
    return complaintController.add(req, res)
   case 'country':
    return countryController.add(req, res)
   case 'region':
    return regionController.add(req, res)
   case 'city':
    return cityController.add(req, res)
   case 'barangay':
    return barangayController.add(req, res)
   case 'data-option':
    return dataOptionController.add(req, res)
   case 'data-option-type':
    return dataOptionTypeController.add(req, res)
   case 'user':
    return userController.add(req, res)
   case 'user-detail':
    return userDetailController.add(req, res)
   case 'banner':
    return bannerController.add(req, res)
   case 'announcement':
    return announcementController.add(req, res)
   case 'news':
    return newsController.add(req, res)
   case 'team':
    return teamController.add(req, res)
   case 'emergency':
    return emergencyController.add(req, res)
   case 'car-sticker':
    return carStickerController.add(req, res)
   default:
    return res.status(400).send({
     statusCode: 400,
     message: 'Invalid Entity Type'
    })
  }
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
    statusCode: 400,
    message: error
  })
 }
}

const feGet = async (req, res) => {
 try {
  const type = req.params.type
  switch (type) {
   case 'role':
    return roleController.get(req, res)
   case 'complaint':
    return complaintController.get(req, res)
   case 'country':
    return countryController.get(req, res)
   case 'region':
    return regionController.get(req, res)
   case 'city':
    return cityController.get(req, res)
   case 'barangay':
    return barangayController.get(req, res)
   case 'data-option':
    return dataOptionController.get(req, res)
   case 'data-option-type':
    return dataOptionTypeController.get(req, res)
   case 'user':
    return userController.get(req, res)
   case 'user-detail':
    return userDetailController.get(req, res)
   case 'banner':
    return bannerController.get(req, res)
   case 'announcement':
    return announcementController.get(req, res)
   case 'news':
    return newsController.get(req, res)
   case 'team':
    return teamController.get(req, res)
   case 'emergency':
    return emergencyController.get(req, res)
   case 'car-sticker':
    return carStickerController.get(req, res)
   default:
    return res.status(400).send({
     statusCode: 400,
     message: 'Invalid Entity Type'
    })
   }
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
    statusCode: 400,
    message: error
  })
 }
}

const feGetAll = async (req, res) => {
 try {
  const type = req.params.type
  switch (type) {
   case 'role':
    return roleController.getAll(req, res)
   case 'complaint':
    return complaintController.getAll(req, res)
   case 'country':
    return countryController.getAll(req, res)
   case 'region':
    return regionController.getAll(req, res)
   case 'city':
    return cityController.getAll(req, res)
   case 'barangay':
    return barangayController.getAll(req, res)
   case 'data-option':
    return dataOptionController.getAll(req, res)
   case 'data-option-type':
    return dataOptionTypeController.getAll(req, res)
   case 'user':
    return userController.getAll(req, res)
   case 'user-detail':
    return userDetailController.getAll(req, res)
   case 'banner':
    return bannerController.getAll(req, res)
   case 'announcement':
    return announcementController.getAll(req, res)
   case 'news':
    return newsController.getAll(req, res)
   case 'team':
    return teamController.getAll(req, res)
   case 'emergency':
    return emergencyController.getAll(req, res)
   case 'car-sticker':
    return carStickerController.getAll(req, res)
   default:
    return res.status(400).send({
     statusCode: 400,
     message: 'Invalid Entity Type'
    })
   }
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
    statusCode: 400,
    message: error
  })
 }
}

const feGetOne = async (req, res) => {
  try {
   const type = req.params.type
   const id = req.params.id

   if (!id) return res.status(400).send({ statusCode: 400, message: 'Invalid FE ID' })

   if (id) {
    switch (type) {
      case 'role':
       return roleController.getOne(req, res)
      case 'complaint':
       return complaintController.getOne(req, res)
      case 'country':
       return countryController.getOne(req, res)
      case 'region':
       return regionController.getOne(req, res)
      case 'city':
       return cityController.getOne(req, res)
      case 'barangay':
       return barangayController.getOne(req, res)
      case 'data-option':
       return dataOptionController.getOne(req, res)
      case 'data-option-type':
       return dataOptionTypeController.getOne(req, res)
      case 'user':
       return userController.getOne(req, res)
      case 'user-detail':
       return userDetailController.getOne(req, res)
      case 'banner':
       return bannerController.getOne(req, res)
      case 'announcement':
       return announcementController.getOne(req, res)
      case 'news':
       return newsController.getOne(req, res)
      case 'team':
       return teamController.getOne(req, res)
      case 'emergency':
       return emergencyController.getOne(req, res)
      case 'car-sticker':
       return carStickerController.getOne(req, res)
      default:
       return res.status(400).send({
        statusCode: 400,
        message: 'Invalid Entity Type'
       })
      }
   }
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
     statusCode: 400,
     message: error
   })
  }
 }

const feGetAllFiltered = async (req, res) => {
 try {
  const type = req.params.type
  switch (type) {
   case 'role':
    return roleController.getFiltered(req, res)
   case 'complaint':
    return complaintController.getFiltered(req, res)
   case 'country':
    return countryController.getFiltered(req, res)
   case 'region':
    return regionController.getFiltered(req, res)
   case 'city':
    return cityController.getFiltered(req, res)
   case 'barangay':
    return barangayController.getFiltered(req, res)
   case 'data-option':
    return dataOptionController.getFiltered(req, res)
   case 'data-option-type':
    return dataOptionTypeController.getFiltered(req, res)
   case 'user':
    return userController.getFiltered(req, res)
   case 'user-detail':
    return userDetailController.getFiltered(req, res)
   case 'banner':
    return bannerController.getFiltered(req, res)
   case 'announcement':
    return announcementController.getFiltered(req, res)
   case 'news':
    return newsController.getFiltered(req, res)
   case 'team':
    return teamController.getFiltered(req, res)
   case 'emergency':
    return emergencyController.getFiltered(req, res)
   case 'car-sticker':
    return carStickerController.getFiltered(req, res)
   default:
    return res.status(400).send({
     statusCode: 400,
     message: 'Invalid Entity Type'
    })
   }
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
    statusCode: 400,
    message: error
  })
 }
}

const fePut = async (req, res) => {
 try {
  const type = req.params.type
  switch (type) {
   case 'role':
    return roleController.update(req, res)
   case 'complaint':
    return complaintController.update(req, res)
   case 'country':
    return countryController.update(req, res)
   case 'region':
    return regionController.update(req, res)
   case 'city':
    return cityController.update(req, res)
   case 'barangay':
    return barangayController.update(req, res)
   case 'data-option':
    return dataOptionController.update(req, res)
   case 'data-option-type':
    return dataOptionTypeController.update(req, res)
   case 'user':
    return userController.update(req, res)
   case 'user-detail':
    return userDetailController.update(req, res)
   case 'banner':
    return bannerController.update(req, res)
   case 'announcement':
    return announcementController.update(req, res)
   case 'news':
    return newsController.update(req, res)
   case 'team':
    return teamController.update(req, res)
   case 'emergency':
    return emergencyController.update(req, res)
   case 'car-sticker':
    return carStickerController.update(req, res)
   default:
    return res.status(400).send({
     statusCode: 400,
     message: 'Invalid Entity Type'
    })
   }
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
    statusCode: 400,
    message: error
  })
 }
}

const feDelete = async (req, res) => {
 try {
  const type = req.params.type
  switch (type) {
   case 'role':
    return roleController.remove(req, res)
   case 'complaint':
    return complaintController.remove(req, res)
   case 'country':
    return countryController.remove(req, res)
   case 'region':
    return regionController.remove(req, res)
   case 'city':
    return cityController.remove(req, res)
   case 'barangay':
    return barangayController.remove(req, res)
   case 'data-option':
    return dataOptionController.remove(req, res)
   case 'data-option-type':
    return dataOptionTypeController.remove(req, res)
   case 'user':
    return userController.remove(req, res)
   case 'user-detail':
    return userDetailController.remove(req, res)
   case 'banner':
    return bannerController.remove(req, res)
   case 'announcement':
    return announcementController.remove(req, res)
   case 'news':
    return newsController.remove(req, res)
   case 'team':
    return teamController.remove(req, res)
   case 'emergency':
    return emergencyController.remove(req, res)
   case 'car-sticker':
    return carStickerController.remove(req, res)
   default:
    return res.status(400).send({
     statusCode: 400,
     message: 'Invalid Entity Type'
    })
   }
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
    statusCode: 400,
    message: error
  })
 }
}

const feInitialize = async (req, res) => {
  try {
    // SYSTEM SETUP WILL GO RIGHT HERE
    return res.status(200).send('Authenticated')
  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

const evaluateToken = async (req, res) => {
  try {
    const token = await req.params.token

    verify(token, process.env.ACCESS_TOKEN_SECRET, (err, user) => {
      if (err) return res.sendStatus(401)
      loginController.doLogin(user)
        .then((response) => {
          return res.status(200).send(response)
        })
    })
  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

const evaluateRefreshToken = async (req, res) => {
  try {
    const token = await req.params.token
    if (!token) { return res.sendStatus(401) }
    return reEvaluateRefreshToken(token, res)
  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

const sendCustomEmail = async (req, res) => {
  try {
    return mailController.sendCustomEmail(req, res)
  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

const sendRejectionEmail = async (req, res) => {
  try {
    return mailController.sendRejectionEmail(req, res)
  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
      statusCode: 400,
      message: error
    })
  }
}

module.exports = {
 fePost,
 feGet,
 feGetAll,
 feGetOne,
 feGetAllFiltered,
 fePut,
 feDelete,
 feInitialize,
 evaluateToken,
 evaluateRefreshToken,
 sendCustomEmail,
 sendRejectionEmail
}