const db = require('../models')
const UAParser = require('ua-parser-js')
const { v4: uuidv4 } = require('uuid')
const mailController = require('./mail.controller')

const Inquiry = db.inquiries

const addNewInquiry = async (req, res) => {
    try {
        const userAgent = req.headers['user-agent']
        const ipAddress = req.ip
        const parser = new UAParser();
        const result = parser.setUA(userAgent).getResult();

        let toSave = {
            id: uuidv4(),
            name: req.body.name,
            subject: req.body.subject,
            email: req.body.email,
            details: req.body.details,
            ipAddress: ipAddress,
            browser: JSON.stringify(result.browser),
            engine: JSON.stringify(result.engine),
            os: JSON.stringify(result.os),
            device: JSON.stringify(result.device)
        }
        Inquiry.create(toSave)
            .then(() => {
                return  mailController.inquiryEmail(res, toSave)
            }).catch((error) => {
                return res.status(400).send({
                  statusCode: 400,
                  message: error
                })
              })
    } catch (error) {
        // Handle errors
        console.error('Error:', error.message);
        res.status(500).json({ error: 'Internal Server Error' });
    }
}

module.exports = {
    addNewInquiry
}