require('dotenv').config()
const { generateTokens } = require('../services/authenticate.service')
const loginService = require('../services/login.service')
const userController = require('../controllers/user.controller')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const bcrypt = require('bcrypt')

const login = async (req, res) => {
  await bcrypt.hash(req.body.password, 10)
      .then((result) => {
        console.info('Password Hash', result)
      })
  const result = await loginService.loginRules(req, res)

  if (result) {
    if (result.code === 'ERROR') {
      return res.status(400).send({
        statusCode: 400,
        message: 'Unable to process your login request.'
      })
    } else {
      if (result.code === 'SUCCESS') {
        const userEmail = req.body.email
        const userPassword = req.body.password
        const userId = result.content && result.content.id
        const userRole = result.content && result.content.role
        const user = { id: userId, email: userEmail, password: userPassword, role: userRole }
        const tokens = generateTokens(user)

        return res.status(201).send({
          statusCode: result.statusCode,
          type: result.code,
          message: result.message,
          content: {
            ...result.content,
            accessToken: tokens.accessToken,
            refreshToken: tokens.refreshToken
          }
        })
      } else {
        return res.status(200).send({
          statusCode: result.statusCode,
          type: result.code,
          message: result.message
        })
      }
    }
  } else {
    return res.status(400).send({
      statusCode: 400,
      message: 'No response'
    })
  }
}

const logout = async (req, res) => {
  // Set Access Token Cookie
  res.cookie('accessToken', '', {
    maxAge: 300000,
    httpOnly: true
  })

  // Set Refresh Token Cookie
  res.cookie('refreshToken', '', {
    maxAge: 3.154e10,
    httpOnly: true
  })

  return res.status(201).send({
    statusCode: result.statusCode,
    type: result.code,
    message: result.message,
    content: {
      token: '',
      email: req.body.email
    }
  })
}

const doLogin = async (user) => {
  const result = await userController.findUserByEmailFnReturn(user)
  return result
}

module.exports = {
  login,
  logout,
  doLogin
}