const nodemailer = require('nodemailer')
const handlebars = require('handlebars')
const path = require('path')
const fs = require('fs')
const moment = require('moment')
const db = require('../models')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

const User = db.users
const UserDetail = db.userDetails
const DataOption = db.dataOptions

const assignedStaff = [
  {
    name: 'Eleanor Concepcion',
    email: 'eleanorconcepcion17@gmail.com',
    availability: [1, 2, 3, 4, 5, 6, 7],
    category: ['request']
  },
  {
    name: 'Noralyn Feliciano',
    email: 'flicianonoralyn3@gmail.com',
    availability: [1, 2, 3, 4, 5, 6, 7],
    category: ['request']
  },
  {
    name: 'Jovelle Santos',
    email: 'jovellequiambao@yahoo.com',
    availability: [1, 2, 3, 4, 5, 6, 7],
    category: ['request']
  },
  {
    name: 'Khristine Apilado',
    email: 'ttugade08@gmail.com',
    availability: [1, 2, 3, 4, 5, 6, 7],
    category: ['request']
  },
  {
    name: 'Orly',
    email: 'oly_gelito@yahoo.com',
    availability: [1, 2, 3, 4, 5, 6, 7],
    category: ['car-sticker']
  },
  {
    name: 'Kag. Austin',
    email: 'victoremmanuelfrancisco@yahoo.com',
    availability: [1, 2, 3, 4, 5, 6, 7],
    category: ['complaint', 'request', 'qerr', 'car-sticker', 'inquiry']
  },
  {
    name: 'Kag. Vener',
    email: 'morinaga0103@gmail.com',
    availability: [1, 2, 3, 4, 5, 6, 7],
    category: ['complaint', 'request', 'qerr', 'car-sticker', 'inquiry']
  },
  {
    name: 'Kag. Ace',
    email: 'aceraymundo@gmail.com',
    availability: [1, 2, 3, 4, 5, 6, 7],
    category: ['qerr']
  },
  {
    name: 'Kap. Dondon',
    email: 'donvicsf@yahoo.com',
    availability: [1, 2, 3, 4, 5, 6, 7],
    category: ['complaint', 'qerr', 'inquiry']
  },
  {
    name: 'Roniell Perez',
    email: 'perezjannroniell@gmail.com',
    availability: [1, 2, 3, 4, 5, 6, 7],
    category: ['qerr']
  }
]

let transporter = nodemailer.createTransport({
  service: 'gmail',
  secure: true,
  port: 25,
  auth: {
    user: config.email,
    pass: config.pass,
  },
  tls: {
   rejectUnauthorized: false
  },
  debug: true
})

const emailUserRegistrationReceived = async (userDetails) => {
  try {

   const filePath = path.join(__dirname, '../public/templates/user-registration-received.html');
   const source = fs.readFileSync(filePath, 'utf-8').toString();
   const template = handlebars.compile(source);

   let user = await User.findOne({
    where: { brgyKey: userDetails.loginId }
   })

   const htmlToSend = template(userDetails.dataValues);

   let mailOptions = {
     from: `Barangay Namayan <${config.email}>`,
     to: user.email,
     subject: 'We received your registration to 1Namayan App',
     html: htmlToSend
   }

   transporter.sendMail(mailOptions)
   log.info(`Registration Email Sent to ${user.email}`)

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const emailUserRegistrationApproved = async (userId) => {
  try {

   const filePath = path.join(__dirname, '../public/templates/user-registration-approved.html.html');
   const source = fs.readFileSync(filePath, 'utf-8').toString();
   const template = handlebars.compile(source);

   let user = await User.findOne({
    where: {id: userId}
   })

   let userDetails = await UserDetail.findOne({
    where: {loginId: user.brgyKey}
   })

   const htmlToSend = template(userDetails.dataValues);

   let mailOptions = {
     from: `Barangay Namayan <${config.email}>`,
     to: user.email,
     subject: 'Welcome to 1Namayan App!',
     html: htmlToSend
   }

   transporter.sendMail(mailOptions)
   log.info(`User Approval Email Sent to ${user.email}`)

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const emailComplaintReceived = async (complaint, complaintById) => {
  try {

   const filePath = path.join(__dirname, '../public/templates/complaint-received.html');
   const source = fs.readFileSync(filePath, 'utf-8').toString();
   const template = handlebars.compile(source);

   let userDetails = await UserDetail.findOne({
    where: { brgyKey: complaintById }
   })

   let user = await User.findOne({
    where: { brgyKey: userDetails.loginId }
   })

   let complaintType = await DataOption.findOne({
    where: { brgyKey: complaint.typeId }
   })

   let info = {
    name: userDetails.firstName,
    ticketNumber: complaint.brgyKey,
   }

   const htmlToSend = template(info);

   let mailOptions = {
     from: `Barangay Namayan <${config.email}>`,
     to: user.email,
     subject: 'We received your report via 1Namayan App',
     html: htmlToSend
   }

   transporter.sendMail(mailOptions)
   log.info(`Complaint Email Sent to ${user.email}`)

   let complaintAttachments = []
   let availableStaffs = getAvailableStaff('complaint')

   if (complaint.attachments) {
    let tempAtt = complaint.attachments.split(',')
    tempAtt.forEach((att, i) => {
      let extension = att.split('.').pop()
      complaintAttachments.push({
        filename: `complaint${i + 1}.${extension}`,
        path: `https://www.1namayan.com/${att}`
      })
    })
   }

   availableStaffs.forEach(staff => {
    let staffInfo = {
      name: staff.name,
      email: staff.email,
      ticketNumber: complaint.brgyKey,
      category: complaintType.name,
      subject: complaint.subject,
      details: complaint.description,
      attachments: complaintAttachments && complaintAttachments.length > 0 ? 'See Attachment(s) Below' : 'N/A'
     }
     if (staffInfo.email) {
      emailComplaintToStaff(staffInfo, complaintAttachments)
     }
   })

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const emailComplaintToStaff = async (staffInfo, complaintAttachments) => {
  try {
   const filePath = path.join(__dirname, '../public/templates/complaint-staff.html');
   const source = fs.readFileSync(filePath, 'utf-8').toString();
   const template = handlebars.compile(source);

   const htmlToSend = template(staffInfo);

   let mailOptions = {
     from: `Barangay Namayan <${config.email}>`,
     to: staffInfo.email,
     subject: `Report Received via 1NApp_${staffInfo.ticketNumber}`,
     html: htmlToSend
   }

   if (complaintAttachments && complaintAttachments.length > 0) {
      mailOptions.attachments = complaintAttachments
   }

   await transporter.sendMail(mailOptions)
   log.info(`Complaint Sent to (${staffInfo.name} - ${staffInfo.email})`)

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const emailQerrReceived = async (details, emergencyBy) => {
  try {

   const filePath = path.join(__dirname, '../public/templates/qerr-received.html');
   const source = fs.readFileSync(filePath, 'utf-8').toString();
   const template = handlebars.compile(source);

   let user = await User.findOne({
    where: { brgyKey: emergencyBy.loginId }
   })

   let info = {
    name: emergencyBy.firstName,
    ticketNumber: details.brgyKey
   }

   const htmlToSend = template(info);

   let mailOptions = {
     from: `Barangay Namayan <${config.email}>`,
     to: user.email,
     subject: 'We received your Quick Emergency Response Request (QERR) via 1Namayan App',
     html: htmlToSend
   }

   transporter.sendMail(mailOptions)
   log.info(`Registration Email Sent to ${user.email}`)

   let availableStaffs = getAvailableStaff('qerr')

   const address = getAddress(emergencyBy)

   availableStaffs.forEach(staff => {
    let staffInfo = {
      name: staff.name,
      email: staff.email,
      ticketNumber: details.brgyKey,
      category: 'Emergency',
      ambulance: details.requestForAmbulance ? 'Yes' : 'No',
      firstaid: details.requestForFirstAid ? 'Yes' : 'No',
      fire: details.requestForFireBrigade ? 'Yes' : 'No',
      tanod: details.requestForTanodPolice ? 'Yes' : 'No',
      other: details.others ? 'Yes' : 'No',
      location: details.homeAddress ? address : details.location,
      details: details.description,
      contact: emergencyBy.mobile,
      person: `${emergencyBy.firstName} ${emergencyBy.lastName}`
     }
     if (staffInfo.email) {
      emailQerrResolved(staffInfo)
     }
   })

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const emailQerrResolved = async (staff) => {
  try {

   const filePath = path.join(__dirname, '../public/templates/qerr-staff.html');
   const source = fs.readFileSync(filePath, 'utf-8').toString();
   const template = handlebars.compile(source);

   const htmlToSend = template(staff);

   let mailOptions = {
     from: `Barangay Namayan <${config.email}>`,
     to: staff.email,
     subject: `URGENT QERR Received via 1NApp_${staff.ticketNumber}`,
     html: htmlToSend
   }

   transporter.sendMail(mailOptions)
   log.info(`Emergency Email Sent to (${staff.name} - ${staff.email})`)

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const getAvailableStaff = (type) => {
  let staffs = []
  assignedStaff.forEach((staff, i) => {
    let today = new Date()
    let todayIndex = moment(today).day()
    if (staff.availability.includes(todayIndex) && staff.category.includes(type)) {
      staffs.push(staff)
    }
  })
  return staffs
}

const getAddress = (entity) => {
  const parts = [
      entity.unitNumber ? entity.unitNumber : '',
      entity.street ? entity.street.name : '',
      entity.barangay ? entity.barangay.name : '',
      entity.userCity ? entity.userCity.name : '',
      entity.userRegion ? entity.userRegion.code : '',
      entity.userCountry ? entity.userCountry.code : ''
  ]

  // Filter out empty values and join the parts with a space
  const address = parts.filter(Boolean).join(' ').trim()

  return address ? address : 'No Address'
}

const getName = (entity) => {
  return entity.suffix
        ? entity.middleName
          ? `${entity.firstName} ${entity.middleName} ${entity.lastName} ${entity.suffix}`
          : `${entity.firstName} ${entity.lastName} ${entity.suffix}`
        : entity.middleName
          ? `${entity.firstName} ${entity.middleName} ${entity.lastName}`
          : `${entity.firstName} ${entity.lastName}`
}

const sendCustomEmail = (req, res) => {
  try {
    const filePath = path.join(__dirname, '../public/templates/custom-email.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);

    let entity = req.body

    const htmlToSend = template(entity);

    let mailOptions = {
      from: `${entity.fromName} <${entity.fromEmail}>`,
      to: entity.toEmail,
      subject: `${entity.subject}`,
      html: htmlToSend
    }

    transporter.sendMail(mailOptions)
    log.info(`Custom Email Sent to ${entity.toEmail}`)

    return res.status(200).send(entity)
  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendRejectionEmail = (req, res) => {
  try {
    const filePath = path.join(__dirname, '../public/templates/rejection-email.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);

    let entity = req.body

    const htmlToSend = template(entity);

    let mailOptions = {
      from: `${entity.fromName} <${entity.fromEmail}>`,
      to: entity.toEmail,
      subject: `${entity.subject}`,
      html: htmlToSend
    }

    transporter.sendMail(mailOptions)
    log.info(`Custom Email Sent to ${entity.toEmail}`)

    return res.status(200).send(entity)
  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendRequestDocumentInitialEmail = (request) => {
  try {
    let converted = JSON.stringify(request)
    let entity = JSON.parse(converted)

    const filePath = path.join(__dirname, '../public/templates/request-document-initial.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);

    if (entity && !entity.requestedBy && !entity.requestedBy.user) { return }

    let info = {
      name: getName(entity.requestedBy),
      address: getAddress(entity.requestedBy),
      email: entity.requestedBy && entity.requestedBy.user.email,
      category: entity.documentCategory && entity.documentCategory.name,
      ticketNumber: entity.ticketNumber
    }

    const htmlToSend = template(info);

    let mailOptions = {
      from: `Barangay Namayan <${config.email}>`,
      to: info.email,
      subject: `We received your request for ${info.category}`,
      html: htmlToSend
    }

    transporter.sendMail(mailOptions)
    log.info(`Request Document (Initial) Email Sent to ${info.email}`)

    let availableStaffs = getAvailableStaff('request')

    let photoAttachments = []

    if (entity.userPhoto != null || entity.cedulaPhoto != null) {
        if (entity.userPhoto != null) {
          let extension = entity.userPhoto.split('.').pop()
          photoAttachments.push({
            filename: `userPhoto.${extension}`,
            path: `https://www.1namayan.com/${entity.userPhoto}`
          })
        }
      
        if (entity.cedulaPhoto != null) {
          let extension = entity.cedulaPhoto.split('.').pop()
          photoAttachments.push({
            filename: `cedulaPhoto.${extension}`,
            path: `https://www.1namayan.com/${entity.cedulaPhoto}`
          })
        }
    }

    availableStaffs.forEach(staff => {
      let staffInfo = {
        name: staff.name,
        email: staff.email,
        category: entity.documentCategory && entity.documentCategory.name,
        ticketNumber: entity.ticketNumber,
        requestedByName: info.name,
        requestedByAddress: info.address,
        requestStay: entity.yearsOfStay && entity.monthsOfStay ? `${entity.yearsOfStay} years and ${entity.monthsOfStay} months` : null,
        requestPurpose: entity.purpose ? entity.purpose.code === 'OTHERS_PURPOSE' ? entity.purposeOthers : entity.purpose && entity.purpose.name : null,
        requestCedula: entity.cedulaPhoto !== null ? `Please see attached file` : null,
        requestRemarks: entity.remarks || null,
        requestedByMobileNumber: entity.requestedBy.mobile || null,
        requestedByCivilStatus: entity.civilStatus && entity.civilStatus.name || null,
        requestedByAnnualIncome: entity.annualIncome || null,
        requestedByCitizenship: entity.citizenship || null
      }
      if (staffInfo.email) {
        sendRequestDocumentInitialEmailStaff(staffInfo, photoAttachments)
      }
    })

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendRequestDocumentInitialEmailStaff = async (staff, photoAttachments) => {
  try {

    const filePath = path.join(__dirname, '../public/templates/request-document-initial-staff.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);

    const htmlToSend = template(staff);

    let mailOptions = {
      from: `Barangay Namayan <${config.email}>`,
      to: staff.email,
      subject: `Service Request Received via 1NApp_${staff.ticketNumber}`,
      html: htmlToSend
    }

    if (photoAttachments && photoAttachments.length > 0) {
      mailOptions.attachments = photoAttachments
    }

    await transporter.sendMail(mailOptions)
    log.info(`Request Document Sent to (${staff.name} - ${staff.email})`)

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendRequestDocumentReleaseEmail = (request) => {
  try {
    let converted = JSON.stringify(request)
    let entity = JSON.parse(converted)

    const filePath = path.join(__dirname, '../public/templates/request-document-release.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);

    if (entity && !entity.requestedBy && !entity.requestedBy.user) { return }

    let info = {
      name: getName(entity.requestedBy),
      address: getAddress(entity.requestedBy),
      email: entity.requestedBy && entity.requestedBy.user.email,
      category: entity.documentCategory && entity.documentCategory.name,
      ticketNumber: entity.ticketNumber
    }

    const htmlToSend = template(info);

    let mailOptions = {
      from: `Barangay Namayan <${config.email}>`,
      to: info.email,
      subject: `Your ${info.category} is ready for release`,
      html: htmlToSend
    }

    transporter.sendMail(mailOptions)
    log.info(`Request Document (Release) Email Sent to ${info.email}`)
  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendRequestDocumentCancelledEmail = (request) => {
  try {
    let converted = JSON.stringify(request)
    let entity = JSON.parse(converted)

    const filePath = path.join(__dirname, '../public/templates/request-car-sticker-cancelled.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);

    if (entity && !entity.carStickerBy && !entity.carStickerBy.user) { return }

    let info = {
      name: getName(entity.carStickerBy),
      address: getAddress(entity.carStickerBy),
      email: entity.carStickerBy && entity.carStickerBy.user.email
    }

    const htmlToSend = template(info);

    let mailOptions = {
      from: `Barangay Namayan <${config.email}>`,
      to: info.email,
      subject: `Your Car Sticker Request has been cancelled`,
      html: htmlToSend
    }

    transporter.sendMail(mailOptions)
    log.info(`Request Car Sticker (Cancelled) Email Sent to ${info.email}`)

    let availableStaffs = getAvailableStaff('request')

    availableStaffs.forEach(staff => {
      let staffInfo = {
        name: staff.name,
        email: staff.email,
        vehicleModel: entity.vehicleModel,
        vehiclePlateNumber: entity.vehiclePlateNumber,
        requestedByName: info.name,
        requestedByAddress: info.address
      }
      if (staffInfo.email) {
        sendRequestDocumentCancelledEmailStaff(staffInfo)
      }
    })

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendRequestDocumentCancelledEmailStaff = async (staff) => {
  try {

   const filePath = path.join(__dirname, '../public/templates/request-car-sticker-cancelled-staff.html');
   const source = fs.readFileSync(filePath, 'utf-8').toString();
   const template = handlebars.compile(source);

   const htmlToSend = template(staff);

   let mailOptions = {
     from: `Barangay Namayan <${config.email}>`,
     to: staff.email,
     subject: `Car Sticker Service Request Cancelled (${staff.vehiclePlateNumber})`,
     html: htmlToSend
   }

   transporter.sendMail(mailOptions)
   log.info(`Request Car Sticker (Cancelled) Sent to (${staff.name} - ${staff.email})`)

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendCarStickerInitialEmail = (request) => {
  try {
    let converted = JSON.stringify(request)
    let entity = JSON.parse(converted)

    const filePath = path.join(__dirname, '../public/templates/request-car-sticker-initial.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);

    if (entity && !entity.carStickerBy && !entity.carStickerBy.user) { return }

    let info = {
      name: getName(entity.carStickerBy),
      address: getAddress(entity.carStickerBy),
      email: entity.carStickerBy && entity.carStickerBy.user.email
    }

    const htmlToSend = template(info);

    let mailOptions = {
      from: `Barangay Namayan <${config.email}>`,
      to: info.email,
      subject: `We received your request for Car Sticker`,
      html: htmlToSend
    }

    transporter.sendMail(mailOptions)
    log.info(`Request Car Sticker (Initial) Email Sent to ${info.email}`)

    let photoAttachments = []

    if (entity.orcr || entity.vehiclePhoto) {
      if (entity.orcr) {
          const orcrExtension = entity.orcr.split('.').pop();
          photoAttachments.push({
              filename: `orcr.${orcrExtension}`,
              path: `https://www.1namayan.com/${entity.orcr}`
          });
      }
  
      if (entity.vehiclePhoto) {
          const vehiclePhotoExtension = entity.vehiclePhoto.split('.').pop();
          photoAttachments.push({
              filename: `vehiclePhoto.${vehiclePhotoExtension}`,
              path: `https://www.1namayan.com/${entity.vehiclePhoto}`
          });
      }
  }

    let availableStaffs = getAvailableStaff('car-sticker')

    console.log('availableStaffs', availableStaffs.length)

    availableStaffs.forEach(staff => {
      let staffInfo = {
        name: staff.name,
        email: staff.email,
        vehicleModel: entity.vehicleModel,
        vehiclePlateNumber: entity.vehiclePlateNumber,
        requestedByName: info.name,
        requestedByAddress: info.address
      }

      console.log('staffInfo', staffInfo)
      if (staffInfo.email) {
        sendCarStickerInitialEmailStaff(staffInfo, photoAttachments)
      }
    })

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendCarStickerInitialEmailStaff = async (staff, photoAttachments) => {
  try {

    const filePath = path.join(__dirname, '../public/templates/request-car-sticker-initial-staff.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);

    const htmlToSend = template(staff);

    let mailOptions = {
      from: `Barangay Namayan <${config.email}>`,
      to: staff.email,
      subject: `Service Request Received for Car Sticker ${staff.vehiclePlateNumber}`,
      html: htmlToSend
    }

    if (photoAttachments && photoAttachments.length > 0) {
      mailOptions.attachments = photoAttachments
    }

    await transporter.sendMail(mailOptions)
    log.info(`Request Car Sticker Sent to (${staff.name} - ${staff.email})`)

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendCarStickerReleaseEmail = (request) => {
  try {
    let converted = JSON.stringify(request)
    let entity = JSON.parse(converted)

    const filePath = path.join(__dirname, '../public/templates/request-car-sticker-release.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);

    if (entity && !entity.carStickerBy && !entity.carStickerBy.user) { return }

    let info = {
      name: getName(entity.carStickerBy),
      address: getAddress(entity.carStickerBy),
      email: entity.carStickerBy && entity.carStickerBy.user.email
    }

    const htmlToSend = template(info);

    let mailOptions = {
      from: `Barangay Namayan <${config.email}>`,
      to: info.email,
      subject: `Your Car Sticker is ready for release`,
      html: htmlToSend
    }

    transporter.sendMail(mailOptions)
    log.info(`Request Car Sticker (Release) Email Sent to ${info.email}`)
  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendCarStickerCancelledEmail = (request) => {
  try {
    let converted = JSON.stringify(request)
    let entity = JSON.parse(converted)

    const filePath = path.join(__dirname, '../public/templates/request-car-sticker-cancelled.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);

    if (entity && !entity.carStickerBy && !entity.carStickerBy.user) { return }

    let info = {
      name: getName(entity.carStickerBy),
      address: getAddress(entity.carStickerBy),
      email: entity.carStickerBy && entity.carStickerBy.user.email
    }

    const htmlToSend = template(info);

    let mailOptions = {
      from: `Barangay Namayan <${config.email}>`,
      to: info.email,
      subject: `Your Car Sticker Request has been cancelled`,
      html: htmlToSend
    }

    transporter.sendMail(mailOptions)
    log.info(`Request Document (Cancelled) Email Sent to ${info.email}`)

    let availableStaffs = getAvailableStaff('car-sticker')

    availableStaffs.forEach(staff => {
      let staffInfo = {
        name: staff.name,
        email: staff.email,
        vehicleModel: entity.vehicleModel,
        vehiclePlateNumber: entity.vehiclePlateNumber,
        requestedByName: info.name,
        requestedByAddress: info.address
      }
      if (staffInfo.email) {
        sendCarStickerCancelledEmailStaff(staffInfo)
      }
    })

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendCarStickerCancelledEmailStaff = async (staff) => {
  try {

   const filePath = path.join(__dirname, '../public/templates/request-car-sticker-cancelled-staff.html');
   const source = fs.readFileSync(filePath, 'utf-8').toString();
   const template = handlebars.compile(source);

   const htmlToSend = template(staff);

   let mailOptions = {
     from: `Barangay Namayan <${config.email}>`,
     to: staff.email,
     subject: `Service Request Cancelled for Car Sticker (${staff.vehiclePlateNumber})`,
     html: htmlToSend
   }

   transporter.sendMail(mailOptions)
   log.info(`Request Car Sticker (Cancelled) Sent to (${staff.name} - ${staff.email})`)

  } catch (error) {
    log.fatal(error)
    return error
  }
}

const sendOTPByEmail = async (info) => {
  try {
    const filePath = path.join(__dirname, '../public/templates/email-otp.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);
 
    const htmlToSend = template(info);
 
    let mailOptions = {
      from: `Barangay Namayan <${config.email}>`,
      to: info.email,
      subject: `FOR YOUR ACTION: Isang Namayan App Email Verification`,
      html: htmlToSend
    }
 
    transporter.sendMail(mailOptions)
    log.info(`OTP Sent to (${info.firstName} ${info.lastName} - ${info.email})`)
 
   } catch (error) {
     log.fatal(error)
     return error
   }
}

const requestResetPassword = (res, entity, newPassword) => {
  try {
    const filePath = path.join(__dirname, '../public/templates/reset-password.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);

    const htmlToSend = template({ newPassword });

    let mailOptions = {
      from: `Barangay Namayan <${config.email}>`,
      to: entity.email,
      subject: `Your new 1Namayan App Password is ready.`,
      html: htmlToSend
    }

    transporter.sendMail(mailOptions)
    log.info(`Reset Password sent to ${entity.email}`)

    return res.status(200).send(entity)
  } catch (error) {
    log.fatal(error)
    return error
  }
}

const inquiryEmail = (res, entity) => {
  try {
    const filePath = path.join(__dirname, '../public/templates/inquiry-email.html');
    const source = fs.readFileSync(filePath, 'utf-8').toString();
    const template = handlebars.compile(source);
    const htmlToSend = template(entity);

    let availableStaffs = getAvailableStaff('inquiry')

    availableStaffs.forEach(async (recipient) => {
      let mailOptions = {
        from: `Barangay Namayan <${config.email}>`,
        to: recipient.email,
        subject: `Contact Form - Re: ${entity.subject}`,
        html: htmlToSend
      }
    
      try {
        await transporter.sendMail(mailOptions);
        console.log(`Inquiry Email sent to ${recipient}`);
      } catch (error) {
        console.error(`Error sending inquiry email to ${recipient}:`, error);
      }
    })

    return res.status(200).send(entity)
  } catch (error) {
    log.fatal(error)
    return error
  }
}

module.exports = {
  emailUserRegistrationReceived,
  emailUserRegistrationApproved,
  emailComplaintReceived,
  emailComplaintToStaff,
  emailQerrReceived,
  emailQerrResolved,
  getAvailableStaff,
  getAddress,
  getName,
  sendCustomEmail,
  sendRequestDocumentInitialEmail,
  sendRequestDocumentReleaseEmail,
  sendRequestDocumentCancelledEmail,
  sendCarStickerInitialEmail,
  sendCarStickerReleaseEmail,
  sendCarStickerCancelledEmail,
  sendRejectionEmail,
  sendOTPByEmail,
  requestResetPassword,
  inquiryEmail
}