const db = require('../models')
const bcrypt = require('bcrypt')
const mailController = require('./mail.controller')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// Create Main Model

const User = db.users
const Role = db.roles
 
 // Update Password By User
 const updateUser = async (req, res) => {
  try {
    let userId = req.body.id
    let info = {}

    let user = await User.findOne({
     where: { id: userId }
    })

    if (!user) return req.status(401).send()
    
    const valid = await bcrypt.compare(req.body.oldPassword, user.password)

    if (!valid) return res.status(400).send({ statusCode: 400, message: 'Unable to process your password reset request.' })

    await bcrypt.hash(req.body.newPassword, 10)
    .then((result) => {
      info = {
        password: result,
        oldPassword: user.password,
        updatePassword: true
      }
    })

    await User.update(info, { where: { id: userId } })
      .then((user) => {
      return res.status(200).send(user)
      }).catch((error) => {
        log.fatal(error)
        return res.status(400).send({
          statusCode: 400,
          message: error
        })
      })

   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
 }

 // Update Password By Admin
 const updateAdmin = async (req, res) => {
  try {
    let userId = req.body.id
    let info = {}

    let user = await User.findOne({
     where: { id: userId }
    })

    if (!user) return req.status(401).send()

    await bcrypt.hash(req.body.newPassword, 10)
    .then((result) => {
      info = {
        password: result,
        oldPassword: user.password,
        updatePassword: true
      }
    })

    await User.update(info, { where: { id: userId } })
      .then((user) => {
      return res.status(200).send(user)
      }).catch((error) => {
        log.fatal(error)
        return res.status(400).send({
          statusCode: 400,
          message: error
        })
      })

   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
 }

 const resetPassword = async (req, res) => {
  try {
    let email = req.body.email
    let info = {}
    let newPassword = await generateRandomString()

    let user = await User.findOne({
     where: { email: email }
    })

    if (!user) return res.status(400).send({ statusCode: 400, message: 'Unable to process your password reset request. Invalid User.' })

    let userDet = user.dataValues

    await bcrypt.hash(newPassword, 10)
    .then((result) => {
      info = {
        password: result,
        oldPassword: user.password,
        updatePassword: true
      }
    })

    await User.update(info, { where: { id: userDet.id } })
      .then(() => {
      return mailController.requestResetPassword(res, userDet, newPassword)
      }).catch((error) => {
        log.fatal(error)
        return res.status(400).send({
          statusCode: 400,
          message: error
        })
      })
   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
 }

 const generateRandomString = async () => {
  const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789'
  let randomString = ''

  for (let i = 0; i < 8; i++) {
      const randomIndex = Math.floor(Math.random() * characters.length)
      randomString += characters.charAt(randomIndex)
  }

  return randomString && randomString.toString()
}

module.exports = {
 updateAdmin,
 updateUser,
 resetPassword,
 generateRandomString
}