const config = require('../config')[process.env.NODE_ENV || 'development']
const { Op } = require('sequelize')
const db = require('../models')
var https = require('follow-redirects').https

const UserDetail = db.userDetails
const options = {
    'method': 'POST',
    'hostname': '8glye9.api.infobip.com',
    'path': '/sms/2/text/advanced',
    'headers': {
        'Authorization': 'App ' + config.infobipAuthorizationKey,
        'Content-Type': 'application/json',
        'Accept': 'application/json'
    },
    'maxRedirects': 20
}

const sendEntityMessage = async (entity) => {
    try {
        let to = entity.mobile
        const text = entity.text

        // Check and format the number
        if (to.startsWith('09')) {
            to = '+63' + to.substring(1) // Replace '09' with '+63'
        }  else if (to.startsWith('9')) {
            to = '+63' + to // Add '+63' when nmber starts with '9'
        } else if (to.startsWith('63')) {
            to = '+63' + to.substring(2) // Replace '63' with '+63'
        } 

        // Validate that the number now follows the +63XXXXXXXXXX format
        const regex = /^\+63\d{10}$/
        if (!regex.test(to)) {
            console.error('Invalid number format. Number must start with +63 followed by 10 digits.')
        }

        const messages = [{
            'destinations': [{ 'to': to }],
            'from': '1Namayan',
            'text': text
        }]

        if (messages.length > 0) {       
            const sendSmsPromises = messages.map(message => {
                return new Promise((resolve, reject) => {
                    const req = https.request(options, function (entity) {
                        let chunks = []
        
                        entity.on('data', function (chunk) {
                            chunks.push(chunk)
                        })
        
                        entity.on('end', function () {
                            const body = Buffer.concat(chunks).toString()
                            resolve({ status: 'success', response: body })
                        })
        
                        entity.on('error', function (error) {
                            reject({ status: 'error', error: error.message })
                        })
                    })
                    console.log('Sending Message: {}', messages)
                    req.write(JSON.stringify({ messages: [message] }))
                    req.end()
                })
            })
        
            // Wait for all SMS sending promises to complete
            const results = await Promise.all(sendSmsPromises)

            log.info('Message processing complete, message sent to (' + to + ')')
        
            // Return the results
            return results
        } else {
            console.error('An error occurred while processing the request. No messages found!')
        }
    } catch (error) {
        return console.error('An error occurred while processing the request.')
    }
}

const sendMessage = async (req, res) => {
    try {
        let to = req.body.mobile
        const text = req.body.text

        // Check and format the number
        if (to.startsWith('09')) {
            to = '+63' + to.substring(1) // Replace '09' with '+63'
        }  else if (to.startsWith('9')) {
            to = '+63' + to // Add '+63' when nmber starts with '9'
        } else if (to.startsWith('63')) {
            to = '+63' + to.substring(2) // Replace '63' with '+63'
        } 

        // Validate that the number now follows the +63XXXXXXXXXX format
        const regex = /^\+63\d{10}$/
        if (!regex.test(to)) {
            return res.status(400).json({ error: 'Invalid number format. Number must start with +63 followed by 10 digits.' })
        }

        const messages = [{
            'destinations': [{ 'to': to }],
            'from': '1Namayan',
            'text': text
        }]

        if (messages.length > 0) {       
            const sendSmsPromises = messages.map(message => {
                return new Promise((resolve, reject) => {
                    const req = https.request(options, function (res) {
                        let chunks = []
        
                        res.on('data', function (chunk) {
                            chunks.push(chunk)
                        })
        
                        res.on('end', function () {
                            const body = Buffer.concat(chunks).toString()
                            resolve({ status: 'success', response: body })
                        })
        
                        res.on('error', function (error) {
                            reject({ status: 'error', error: error.message })
                        })
                    })
                    console.log('Sending Message: {}', messages)
                    req.write(JSON.stringify({ messages: [message] }))
                    req.end()
                })
            })
        
            // Wait for all SMS sending promises to complete
            const results = await Promise.all(sendSmsPromises)

            log.info('Message processing complete, message sent to (' + to + ')')
        
            // Return the results
            return res.json({
                message: 'Send message processing complete.',
                results
            })
        } else {
            return res.status(500).json({ error: 'An error occurred while processing the request. No messages found!' })
        }
    } catch (error) {
        return res.status(500).json({ error: 'An error occurred while processing the request.' });
    }
}

const sendBlastMessage = async (req, res) => {
    const text = req.body.text // Message content

    try {
        const mobileNumbers = await getValidUserMobileNumbers()
        if (mobileNumbers.length === 0) {
            return res.status(400).json({ error: 'No valid mobile numbers found.' })
        }

        let filteredNumbers = mobileNumbers.map((mobile) => {
            // Check and format the number
            if (mobile.startsWith('09')) {
                return '+63' + mobile.substring(1) // Replace '09' with '+63'
            }  else if (mobile.startsWith('9')) {
                return '+63' + mobile // Add '+63' when nmber starts with '9'
            } else if (mobile.startsWith('63')) {
                return '+63' + mobile.substring(2) // Replace '63' with '+63'
            } else {
                return null
            }
        }).filter((mobile) => mobile != null)

        // Prepare the message payload
        const messages = filteredNumbers.map(number => ({
            'destinations': [{ 'to': number }],
            'from': '1Namayan',
            'text': text
        }))

        // Send SMS
        const sendSmsPromises = messages.map(message => {
            return new Promise((resolve, reject) => {
                const req = https.request(options, function (res) {
                    let chunks = [];

                    res.on('data', function (chunk) {
                        chunks.push(chunk)
                    });

                    res.on('end', function () {
                        const body = Buffer.concat(chunks).toString()
                        resolve({ status: 'success', response: body })
                    });

                    res.on('error', function (error) {
                        reject({ status: 'error', error: error.message })
                    });
                });

                req.write(JSON.stringify({ messages: [message] }))
                req.end()
            })
        })

        // Wait for all SMS sending promises to complete
        const results = await Promise.all(sendSmsPromises)

        log.info('Blast message processing complete, message sent to (' + filteredNumbers.length + ') users')

        // Return the results
        return res.json({
            message: 'Blast message processing complete.',
            results
        })
    } catch (error) {
        return res.status(500).json({ error: 'An error occurred while processing the request.' })
    }
}

const getValidUserMobileNumbers = async () => {
    try {
        const users = await UserDetail.findAll({
            where: {
                mobile: {
                    [Op.not]: null
                }
            }
        })
        return users.map(user => user.mobile)
    } catch (error) {
        console.error('Error fetching valid users:', error)
        throw error;
    }
}

module.exports = {
    sendMessage,
    sendEntityMessage,
    sendBlastMessage,
    getValidUserMobileNumbers
}