const db = require('../models')
const { v4: uuidv4 } = require('uuid')
const bcrypt = require('bcrypt')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const mailController = require('./mail.controller')

// Create Main Model

const User = db.users
const Role = db.roles
const UserDetail = db.userDetails

// Create
const add = async (req, res) => {
  try {
    let info = {}
  
    let users = await User.findAll({
      where: { email: req.body.email },
      limit: 1
     })
   
    const result = users.length > 0 ? Object.values(JSON.parse(JSON.stringify(users)))[0] : null
    const defaultRole = await Role.findOne({ where: { code: 'USER' }, limit: 1 })

    if (result) {
      const userDetailExist = await UserDetail.findOne({ where: { loginId: result.brgyKey }, limit: 1 })
      if (userDetailExist) {
        return res.status(400).send({ statusCode: 400, message: 'User already exist' })
      } else {
        return res.status(200).send({
          statusCode: 200,
          message: 'Existing user without details!',
          content: result
        })
      }
    }
  
    await bcrypt.hash(req.body.password, 10)
      .then((result) => {
        info = {
          id: uuidv4(),
          email: req.body.email,
          password: result,
          roleId: req.body.role ? req.body.role.brgyKey : defaultRole ? defaultRole.brgyKey : null,
          loginAttempts: req.body.loginAttempts,
          updatePassword: req.body.updatePassword,
          oldPassword: req.body.oldPassword,
          verified: req.body.verified ? req.body.verified : false,
          disabled: req.body.disabled ? req.body.disabled : false,
          approvedBy: req.body.approvedBy,
          approvedOn: req.body.approvedOn
        }
      })

      await User.create(info, {
        include: [{ model: Role, as: 'role', nested: true}]
      }).then((user) => {
        return res.status(200).send({
          statusCode: 200,
          message: 'Successfully Added!',
          content: user
        })
      }).catch((error) => {
        log.fatal(error)
        return res.status(400).send({
          statusCode: 400,
          message: error
        })
      })

  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
    statusCode: 400,
    message: error
    })
  }
}
 
 // Get All
 const getAll = async (req, res) => {
  try {
   let str = req.body && req.body.str
   let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
 
   let filter = str ? { email: { [Op.substring] : '%' + str + '%' } } : ''
 
   await User.findAll({
     where: { ...filter },
     include: [
       { model: Role, as: 'role', nested: true }
      ],
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
      return returnWithCount(res, filter, {}, response, limit)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

  // Get All Enabled
  const get = async (req, res) => {
    try {
     await User.findAll({
      where: {
       [Op.or]: [
         { disabled: null },
         { disabled: false },
         { deletedAt: null }
       ]
      },
      include: [
       {
         model: Role,
         as: 'role'
       }
      ]
     }).then((user) => {
       return res.status(200).send(user)
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
        })
     })
   
    } catch (error) {
     log.fatal(error)
     return res.status(400).send({
      statusCode: 400,
      message: error
     })
    }
   }
 
 // Get Filtered (Admin)
 const getFiltered = async (req, res) => {
  try {
    let sortBy = req.body.sortBy || 'brgyKey'
    let order = req.body.order || 'ASC'
    let limit = req.body.size ? req.body.size : 20
    let pageCount = req.body.page ? req.body.page : 1
    let limitCount = 0 + (pageCount - 1) * limit
    let page = limitCount > 0 ? limitCount : 0
   let filter = {}
   let innerFilter = {}
 
   if (req.body.filter) {
    for (let prop in req.body.filter) {
      if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
         for (let innerprop in req.body.filter[prop]) {
           innerFilter[innerprop] = req.body.filter[prop][innerprop]
         }
        } else {
          filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
        }
      }
    }
  }
   
   await User.findAll({
     where: { ...filter },
     include: [{ model: Role, as: 'role', nested: true, where: { ...innerFilter } }],
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
      return returnWithCount(res, filter, innerFilter, response, limit)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
     })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get One
 const getOne = async (req, res) => {
  try {
   let userId = req.params.id 

   if (!userId) return res.status(400).send({ statusCode: 400, message: 'Invalid User ID' })

   await User.findOne({
    where: { id: userId },
    include: [{ model: Role, as: 'role' }]
   }).then((user) => {
     return res.status(200).send(user)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Update One
 const update = async (req, res) => {
  try {
    let userId = req.params.id
    
    if (!userId) return res.status(400).send({ statusCode: 400, message: 'Invalid User ID' })

    let info = {
      id: req.params.id,
      email: req.body.email,
      roleId: req.body.role ? req.body.role.brgyKey : defaultRole ? defaultRole.brgyKey : null,
      loginAttempts: req.body.loginAttempts,
      updatePassword: req.body.updatePassword,
      oldPassword: req.body.oldPassword,
      verified: req.body.verified ? req.body.verified : false,
      disabled: req.body.disabled ? req.body.disabled : false,
      approvedBy: req.body.approvedBy,
      approvedOn: req.body.approvedOn
    }

    await User.update(info, { where: { id: userId } })
      .then((user) => {
      return res.status(200).send(user)
      }).catch((error) => {
        log.fatal(error)
        return res.status(400).send({
          statusCode: 400,
          message: error
        })
      })

   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
 }
 
 // Delete One
 const remove = async (req, res) => {
  try {
   let userId = req.params.id

   if (!userId) return res.status(400).send({ statusCode: 400, message: 'Invalid User ID' })

   await User.destroy({
    where: {id: userId}
   }).then(() => {
     return res.status(200).send()
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

// Find User By Email
const findUserByEmail = async (req, res) => {
 try {
  let userEmail = req.body.email
  await User.findAll({
   where: { email: userEmail },
   limit: 1
  }).then((users) => {
   const result = users.length > 0 ? Object.values(JSON.parse(JSON.stringify(users)))[0] : null

   return res.status(200).send(result)
  }).catch((error) => {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Reset Login Attempts
const resetLoginAttempts = async (req, res) => {
 try {
  let userId = req.params.id

  if (!userId) return res.status(400).send({ statusCode: 400, message: 'Invalid User ID' })

  await User.update({
   loginAttempts: 0
  }, { where: { id: userId } })
  .then((user) => {
   return res.status(200).send(user)
  })
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Add Login Attempt
const addLoginAttempts = async (req, res) => {
 try {
  let userId = req.body.id
  await User.update({
   loginAttempts: req.body.count
  }, { where: { id: userId } })
   .then((user) => {
    return res.status(200).send(user)
   })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Disabled Account
const disabledAccount = async (req, res) => {
 try {
  let userId = req.params.id || req.body.id
  
  if (!userId) return res.status(400).send({ statusCode: 400, message: 'Invalid User ID' })

  let count = req.body.count || 0
  await User.update({
   loginAttempts: count,
   disabled: true
  }, { where: { id: userId } })
  .then((user) => {
   return res.status(200).send(user)
  }).catch((error) => {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  })
  
 } catch (error) {
  log.fatal(error)
  return res.status(400).send({
   statusCode: 400,
   message: error
  })
 }
}

// Disabled Account
const rejectedAccount = async (req, res) => {
  try {
   let userId = req.params.id || req.body.id

   if (!userId) return res.status(400).send({ statusCode: 400, message: 'Invalid User ID' })

   let count = req.body.count || 0
   await User.update({
    loginAttempts: count,
    rejected: true,
    rejectionNotes: req.body.rejectionNotes,
    rejectedBy: req.body.rejectedBy
   }, { where: { id: userId } })
   .then((user) => {
    return res.status(200).send(user)
   }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

// Approve Account
const approveAccount = async (req, res) => {
  try {
   let userId = req.params.id || req.body.id

   if (!userId) return res.status(400).send({ statusCode: 400, message: 'Invalid User ID' })
   
   await User.update({
    verified: true,
    approvedBy: req.body.approvedBy,
    approvedOn: req.body.approvedOn,
    rejected: null
   }, { where: { id: userId } })
   .then((user) => {
    mailController.emailUserRegistrationApproved(userId)
    return res.status(200).send(user)
   }).catch((error) => {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

const findUserByEmailFn = async (req, res) => {
 try {
  const userEmail = req.body.email
  let users = await User.findAll({
   where: { email: userEmail },
   include: [{ model: Role, as: 'role' }],
   limit: 1
  })

  const result = users.length > 0 ? Object.values(JSON.parse(JSON.stringify(users)))[0] : null
  return result

 } catch (error) {
  log.fatal('findUserByEmailFn', error)
  return null
 }
}

const findUserByEmailFnReturn = async (user) => {
  try {
   const userEmail = user.email
   let users = await User.findAll({
    where: { email: userEmail },
    include: [{ model: Role, as: 'role' }],
    limit: 1
   })
 
   const result = users.length > 0 ? Object.values(JSON.parse(JSON.stringify(users)))[0] : null
   return result
 
  } catch (error) {
   log.fatal('findUserByEmailFnReturn', error)
   return null
  }
 }

const resetLoginAttemptsFn = async (req, res) => {
 try {
  const request = req.body
  let userId = request.id
  await User.update({
   loginAttempts: 0
  }, { where: { id: userId } })
  return true
 } catch (error) {
  log.fatal('resetLoginAttemptsFn', error)
  return false
 }
}

const addLoginAttemptsFn = async (req, res) => {
 try {
  const request = req.body
  await User.update({
   loginAttempts: request.count
  }, { where: { id: request.id } })
  return true
 } catch (error) {
  log.fatal('addLoginAttemptsFn', error)
  return false
 }
}

const getOneUserFn = async (req, res) => {
 try {
  let user = await User.findOne({
   where: {id: req.body.id}
  })
  return user
 } catch (error) {
  log.fatal('disablegetOneUserdAccount', error)
  return null
 }
}

const disabledAccountFn = async (req, res) => {
 try {
  let count = req.body.count || 0
  await User.update({
   loginAttempts: count,
   disabled: true
  }, { where: { id: req.body.id } })
  return true
 } catch (error) {
  log.fatal('disabledAccountFn', error)
  return false
 }
}

const returnWithCount = async (res, filter, innerFilter, response, limit) => {
  if (Object.keys(innerFilter).length) {
    User.count({
      where: { ...filter},
      include: [{ model: Role, as: 'role', nested: true, where: { ...innerFilter } }],
    }).then((totalCount) => {
      let result = {
        content: response,
        totalElements: totalCount,
        totalPages: Math.ceil(totalCount/limit),
      }
      return res.status(200).send(result)
    })
  } else {
    User.count({
      where: { ...filter}
    }).then((totalCount) => {
      let result = {
        content: response,
        totalElements: totalCount,
        totalPages: Math.ceil(totalCount/limit),
      }
      return res.status(200).send(result)
    })
  }
 }

module.exports = {
  add,
  getAll,
  get,
  getFiltered,
  getOne,
  update,
  remove,
  findUserByEmail,
  resetLoginAttempts,
  addLoginAttempts,
  disabledAccount,
  rejectedAccount,
  approveAccount,
  findUserByEmailFn,
  resetLoginAttemptsFn,
  addLoginAttemptsFn,
  getOneUserFn,
  disabledAccountFn,
  findUserByEmailFnReturn,
  returnWithCount
}