const db = require('../models')
const mailController = require('./mail.controller')
const { v4: uuidv4 } = require('uuid')
const { Op } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// Create Main Model

const UserDetail = db.userDetails
const DataOption = db.dataOptions
const Country = db.countries
const Region = db.regions
const City = db.cities
const Barangay = db.barangays
const User = db.users

// Create
const add = async (req, res) => {
  try {
    let shouldDefaultValues = false
    const defaultCountry = await Country.findOne({ where: { code: 'PH' } })
    const defaultRegion = await Region.findOne({ where: { code: 'NCR' } })
    const defaultCity = await City.findOne({ where: { code: 'MANDALUYONG' } })
    const defaultBarangay = await Barangay.findOne({ where: { code: 'NAMAYAN' } })

    if (req.body.natureOfStay && (req.body.natureOfStay.code === 'CURRENT' || req.body.natureOfStay.code === 'BUSINESS')) {
      shouldDefaultValues = true
    }

    let info = {
      id: uuidv4(),
      lastName: req.body.lastName,
      firstName: req.body.firstName,
      middleName: req.body.middleName,
      suffix: req.body.suffix,
      birthDate: req.body.birthDate,
      genderId: req.body.gender ? req.body.gender.brgyKey : null,
      unitNumber: req.body.unitNumber,
      // provinceId: req.body.subStreet ? req.body.subStreet.brgyKey : null,
      streetId: req.body.street ? req.body.street.brgyKey : null,
      barangayId: req.body.barangay
        ? req.body.barangay.brgyKey
        : shouldDefaultValues
          ? (defaultBarangay && defaultBarangay.dataValues && defaultBarangay.dataValues.brgyKey) || null
          : null,
      cityId: req.body.userCity
        ? req.body.userCity.brgyKey
        : shouldDefaultValues
          ? (defaultCity && defaultCity.dataValues && defaultCity.dataValues.brgyKey) || null
          : null,
      regionId: req.body.userRegion
        ? req.body.userRegion.brgyKey
        : shouldDefaultValues
          ? (defaultRegion && defaultRegion.dataValues && defaultRegion.dataValues.brgyKey) || null
          : null,
      countryId: shouldDefaultValues
          ? (defaultCountry && defaultCountry.dataValues && defaultCountry.dataValues.brgyKey) || null
          : null,
      natureOfStayId: req.body.natureOfStay ? req.body.natureOfStay.brgyKey : null,
      otherUnitNumber: req.body.otherUnitNumber,
      otherSubStreet: req.body.otherSubStreet,
      otherStreet: req.body.otherStreet,
      otherBarangay: req.body.otherBarangay,
      otherCity: req.body.otherCity,
      otherRegion: req.body.otherRegion,
      otherCountryId: req.body.otherCountry,
      businessName: req.body.businessName,
      businessOwner: req.body.businessName ? true : false,
      mobile: req.body.mobile,
      bric: req.body.bric,
      brgyVoter: req.body.brgyVoter ? req.body.brgyVoter : false,
      precinct: req.body.precinct,
      loginId: req.body.loginId
    }
  
   await UserDetail.create(info)
     .then((userDetails) => {
       mailController.emailUserRegistrationReceived(userDetails)
       return res.status(200).send(userDetails)
     }).catch((error) => {
       log.fatal(error)
       return res.status(400).send({
         statusCode: 400,
         message: error
       })
     })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get All
 const getAll = async (req, res) => {
  try {
   let str = req.body && req.body.str
   let sortBy = req.body.sortBy || 'brgyKey'
  let order = req.body.order || 'ASC'
  let limit = req.body.size ? req.body.size : 20
  let pageCount = req.body.page ? req.body.page : 1
  let limitCount = 0 + (pageCount - 1) * limit
  let page = limitCount > 0 ? limitCount : 0
 
   let filter = str ? {
     [Op.or]: [
       { firstName: { [Op.substring] : '%' + str + '%' } },
       { lastName: { [Op.substring] : '%' + str + '%' } }
     ]
   } : ''
 
   await UserDetail.findAll({
     where: { ...filter },
     include: [
       { model: DataOption, as: 'gender', nested: true },
       { model: DataOption, as: 'street', nested: true },
      //  { model: DataOption, as: 'subStreet', nested: true },
       { model: DataOption, as: 'natureOfStay', nested: true },
       { model: User, as: 'user', nested: true },
       { model: Barangay, as: 'barangay', nested: true },
       { model: City, as: 'userCity', nested: true },
       { model: Region, as: 'userRegion', nested: true },
       { model: Country, as: 'userCountry', nested: true }
      ],
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
      return returnWithCount(res, filter, response, limit, false)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get All Enabled
 const get = async (req, res) => {
  try {
   await UserDetail.findAll({
    where: { deletedAt: null },
    include: [
      { model: DataOption, as: 'gender', nested: true },
      { model: DataOption, as: 'street', nested: true },
      { model: DataOption, as: 'natureOfStay', nested: true },
      { model: User, as: 'user', nested: true },
      { model: Barangay, as: 'barangay', nested: true },
      { model: City, as: 'userCity', nested: true },
      { model: Region, as: 'userRegion', nested: true },
      { model: Country, as: 'userCountry', nested: true }
     ]
   }).then((userDetails) => {
     return res.status(200).send(userDetails)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

// Get All Enabled
const getUnverified = async (req, res) => {
  try {
    await UserDetail.findAll({
    where: { deletedAt: null },
    include: [
      { model: DataOption, as: 'gender', nested: true },
      { model: DataOption, as: 'street', nested: true },
      { model: DataOption, as: 'natureOfStay', nested: true },
      { model: User, as: 'user', nested: true,
        where: {
          [Op.or]: [
            { verified: false },
            { verified: null }
          ],
          [Op.and]: [
            { [Op.or]: [
                { disabled: false },
                { disabled: null }
              ]
            },
            { [Op.or]: [
                { rejected: false },
                { rejected: null }
              ]
            }
          ]
        }
      },
        { model: Barangay, as: 'barangay', nested: true },
        { model: City, as: 'userCity', nested: true },
        { model: Region, as: 'userRegion', nested: true },
        { model: Country, as: 'userCountry', nested: true }
      ],
      order: [['createdAt', 'desc']]
    }).then((userDetails) => {
      return res.status(200).send(userDetails)
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })
  
  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
    statusCode: 400,
    message: error
    })
  }
}

// Get All Enabled
const getUnverifiedAndRejected = async (req, res) => {
  try {
    await UserDetail.findAll({
    where: { deletedAt: null },
    include: [
      { model: DataOption, as: 'gender', nested: true },
      { model: DataOption, as: 'street', nested: true },
      { model: DataOption, as: 'natureOfStay', nested: true },
      { model: User, as: 'user', nested: true, where: {
        [Op.or]: [
          { verified: false },
          { verified: null }
        ],
        [Op.and]: [
          { [Op.or]: [
              { disabled: false },
              { disabled: null }
            ]
          },
          { [Op.or]: [
              { rejected: true },
              { rejected: null }
            ]
          }
        ]
      }},
      { model: Barangay, as: 'barangay', nested: true },
      { model: City, as: 'userCity', nested: true },
      { model: Region, as: 'userRegion', nested: true },
      { model: Country, as: 'userCountry', nested: true }
      ],
    order: [['createdAt', 'desc']]
    }).then((userDetails) => {
      return res.status(200).send(userDetails)
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })
  
  } catch (error) {
    log.fatal(error)
    return res.status(400).send({
    statusCode: 400,
    message: error
    })
  }
}
 
 // Get Filtered (Admin)
 const getFiltered = async (req, res) => {
  try {
    let sortBy = req.body.sortBy || 'brgyKey'
    let order = req.body.order || 'ASC'
    let limit = req.body.size ? req.body.size : 20
    let pageCount = req.body.page ? req.body.page : 1
    let limitCount = 0 + (pageCount - 1) * limit
    let page = limitCount > 0 ? limitCount : 0
    let filter = {}
 
   if (req.body.filter) {
     for (let prop in req.body.filter) {
       if (req.body.filter[prop]) {
        if (typeof req.body.filter[prop] === 'object') {
          for (let innerprop in req.body.filter[prop]) {
            if (innerprop === 'brgyKey') {
              filter[prop + 'Id'] = req.body.filter[prop][innerprop]
            }
           }
         } else {
           if (prop === 'name') {
            filter = {
              [Op.or]: [
                { ['firstName']: { [Op.substring] : '%' + req.body.filter[prop] + '%' } },
                { ['lastName']: { [Op.substring] : '%' + req.body.filter[prop] + '%' } }
              ]
            }
           } else {
            filter[prop] = { [Op.substring] : '%' + req.body.filter[prop] + '%' }
           }
         }
       }
     }
   }
   
   await UserDetail.findAll({
     where: filter ? { ...filter } : {},
     include: [
        { model: DataOption, as: 'gender', nested: true },
        { model: DataOption, as: 'street', nested: true },
        { model: DataOption, as: 'natureOfStay', nested: true },
        { model: Barangay, as: 'barangay', nested: true },
        { model: City, as: 'userCity', nested: true },
        { model: Region, as: 'userRegion', nested: true },
        { model: Country, as: 'userCountry', nested: true },
      ],
     offset: page,
     limit: limit,
     order: [[sortBy, order]]
   }).then((response) => {
      return returnWithCount(res, filter, response, limit, true)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
     })
   })
 
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Get One
 const getOne = async (req, res) => {
  try {
   let userDetailId = req.params.id

   if (!userDetailId) return res.status(400).send({ statusCode: 400, message: 'Invalid User Detail ID' })
 
   await UserDetail.findOne({
    where: { id: userDetailId },
    include: [
      { model: DataOption, as: 'gender', nested: true },
      { model: DataOption, as: 'street', nested: true },
      { model: DataOption, as: 'natureOfStay', nested: true },
      { model: User, as: 'user', nested: true },
      { model: Barangay, as: 'barangay', nested: true },
      { model: City, as: 'userCity', nested: true },
      { model: Region, as: 'userRegion', nested: true },
      { model: Country, as: 'userCountry', nested: true }
     ]
   }).then((userDetail) => {
     return res.status(200).send(userDetail)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

 const getUserDetailByUserBrgyKey = async (req, res) => {
  try {
   let loginBrgyKey = req.params.brgyKey
 
   await UserDetail.findOne({
    where: { brgyKey: loginBrgyKey },
    include: [
      { model: DataOption, as: 'gender', nested: true },
      { model: DataOption, as: 'street', nested: true },
      { model: DataOption, as: 'natureOfStay', nested: true },
      { model: User, as: 'user', nested: true },
      { model: Barangay, as: 'barangay', nested: true },
      { model: City, as: 'userCity', nested: true },
      { model: Region, as: 'userRegion', nested: true },
      { model: Country, as: 'userCountry', nested: true }
     ],
    limit: 1
   }).then((userDetail) => {
     return res.status(200).send(userDetail)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Update One
 const update = async (req, res) => {
  try {
    const shouldDefaultValues = req.body.shouldDefaultValues
    const defaultCountry = await Country.findOne({ where: { code: 'PH' } })
    const defaultRegion = await Region.findOne({ where: { code: 'NCR' } })
    const defaultCity = await City.findOne({ where: { code: 'MANDALUYONG' } })
    const defaultBarangay = await Barangay.findOne({ where: { code: 'NAMAYAN' } })

   let userDetailId = req.params.id

   if (!userDetailId) return res.status(400).send({ statusCode: 400, message: 'Invalid User Detail ID' })

   let info = {
    lastName: req.body.lastName,
    firstName: req.body.firstName,
    middleName: req.body.middleName,
    suffix: req.body.suffix,
    birthDate: req.body.birthDate,
    genderId: req.body.gender ? req.body.gender.brgyKey : null,
    unitNumber: req.body.unitNumber,
    streetId: req.body.street ? req.body.street.brgyKey : null,
    barangayId: req.body.barangay
      ? req.body.barangay.brgyKey
      : shouldDefaultValues
        ? defaultBarangay || null
        : null,
    cityId: req.body.userCity
      ? req.body.userCity.brgyKey
      : shouldDefaultValues
        ? defaultCity || null
        : null,
    regionId: req.body.userRegion
      ? req.body.userRegion.brgyKey
      : shouldDefaultValues
        ? defaultRegion || null
        : null,
    countryId: req.body.userCountry
      ? req.body.userCountry.brgyKey
      : shouldDefaultValues
        ? defaultCountry || null
        : null,
    natureOfStayId: req.body.natureOfStay ? req.body.natureOfStay.brgyKey : null,
    otherUnitNumber: req.body.otherUnitNumber,
    otherSubStreet: req.body.otherSubStreet,
    otherStreet: req.body.otherStreet,
    otherBarangay: req.body.otherBarangay,
    otherCity: req.body.otherCity,
    otherRegion: req.body.otherRegion,
    otherCountryId: req.body.otherCountry,
    businessName: req.body.businessName,
    businessOwner: req.body.businessName ? true : false,
    mobile: req.body.mobile,
    bric: req.body.bric,
    brgyVoter: req.body.brgyVoter ? req.body.brgyVoter : false,
    precinct: req.body.precinct,
    verification: req.body.verification,
    photo: req.body.photo
  }
 
   await UserDetail.update(info, { where: { id: userDetailId } })
   .then((userDetail) => {
     return res.status(200).send(userDetail)
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
     })
   })
   
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }
 
 // Delete One
 const remove = async (req, res) => {
  try {
   let userDetailId = req.params.id

   if (!userDetailId) return res.status(400).send({ statusCode: 400, message: 'Invalid User Detail ID' })

   await UserDetail.destroy({
    where: {id: userDetailId}
   }).then(() => {
     return res.status(200).send()
   }).catch((error) => {
     log.fatal(error)
     return res.status(400).send({
       statusCode: 400,
       message: error
      })
   })
  } catch (error) {
   log.fatal(error)
   return res.status(400).send({
    statusCode: 400,
    message: error
   })
  }
 }

 const uploadVerification = async (req, res) => {
  try {
    let userDetailId = req.body.id
    await UserDetail.update({ verification: req.body.verificationPath }, { where: { id: userDetailId } })
    .then((success) => {
      if (success) {
        UserDetail.findOne({
          where: { id: userDetailId },
          include: [
            { model: DataOption, as: 'gender', nested: true },
            { model: DataOption, as: 'street', nested: true },
            { model: DataOption, as: 'natureOfStay', nested: true },
            { model: User, as: 'user', nested: true },
            { model: Barangay, as: 'barangay', nested: true },
            { model: City, as: 'userCity', nested: true },
            { model: Region, as: 'userRegion', nested: true },
            { model: Country, as: 'userCountry', nested: true }
           ]
         }).then((userDetail) => {
           return res.status(200).send(userDetail)
         }).catch((error) => {
           log.fatal(error)
           return res.status(400).send({
             statusCode: 400,
             message: error
            })
         })
      } else {
        return res.status(400).send({
          statusCode: 400,
          message: 'Uploading Failed...'
        })
      }
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })
   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
 }

 const uploadPhoto  = async (req, res) => {
  try {
    let userDetailId = req.body.id
    await UserDetail.update({ photo: req.body.imagePath }, { where: { id: userDetailId } })
    .then((success) => {
      if (success) {
        UserDetail.findOne({
          where: { id: userDetailId },
          include: [
            { model: DataOption, as: 'gender', nested: true },
            { model: DataOption, as: 'street', nested: true },
            { model: DataOption, as: 'natureOfStay', nested: true },
            { model: User, as: 'user', nested: true },
            { model: Barangay, as: 'barangay', nested: true },
            { model: City, as: 'userCity', nested: true },
            { model: Region, as: 'userRegion', nested: true },
            { model: Country, as: 'userCountry', nested: true }
           ]
         }).then((userDetail) => {
           return res.status(200).send(userDetail)
         }).catch((error) => {
           log.fatal(error)
           return res.status(400).send({
             statusCode: 400,
             message: error
            })
         })
      } else {
        return res.status(400).send({
          statusCode: 400,
          message: 'Uploading Failed...'
        })
      }
    }).catch((error) => {
      log.fatal(error)
      return res.status(400).send({
        statusCode: 400,
        message: error
      })
    })
   } catch (error) {
    log.fatal(error)
    return res.status(400).send({
     statusCode: 400,
     message: error
    })
   }
 }

 const returnWithCount = async (res, filter, response, limit, isAdmin) => {
  if (isAdmin) {
    UserDetail.count({
      where: { ...filter}
    }).then((totalCount) => {
      let result = {
        content: response,
        totalElements: totalCount,
        totalPages: Math.ceil(totalCount/limit),
      }
      return res.status(200).send(result)
    })
  } else {
    UserDetail.count({
      where: { ...filter,
         [Op.and]: [
           { deletedAt: null }
         ]
       }}).then((totalCount) => {
          let result = {
            content: response,
            totalElements: totalCount,
            totalPages: Math.ceil(totalCount/limit),
          }
          return res.status(200).send(result)
      })
  }
 }
 
 module.exports = {
   add,
   getAll,
   get,
   getUnverified,
   getUnverifiedAndRejected,
   getFiltered,
   getOne,
   update,
   remove,
   getUserDetailByUserBrgyKey,
   uploadVerification,
   uploadPhoto,
   returnWithCount
 }