module.exports = (sequelize, DataTypes) => {
  const Complaint = sequelize.define('complaint', {
    id: {
      type: DataTypes.UUID,
      unique: true
    },
    brgyKey: {
      type: DataTypes.INTEGER,
      autoIncrement: true,
      primaryKey: true
    },
    subject: {
      type: DataTypes.STRING,
      allowNull: false,
    },
    typeId: {
      type: DataTypes.INTEGER,
      allowNull: true
    },
    description: {
      type: DataTypes.TEXT('long'),
      allowNull: true
    },
    complaintById: {
      type: DataTypes.INTEGER,
      allowNull: true
    },
    attachments: {
      type: DataTypes.TEXT,
      allowNull: true
    },
    resolved: {
      type: DataTypes.BOOLEAN,
      allowNull: true
    },
    createdAt: {
      type: DataTypes.DATE,
      defaultValue: DataTypes.NOW
    },
    updatedAt: {
      type: DataTypes.DATE,
      allowNull: true
    },
    deletedAt: {
      type: DataTypes.DATE,
      allowNull: true
    },
    resolvedBy: {
      type: DataTypes.STRING,
      allowNull: true
    },
    resolvedOn: {
      type: DataTypes.DATE,
      allowNull: true
    },
    rejected: {
      type: DataTypes.BOOLEAN,
      allowNull: true
    },
    rejectionNotes: {
      type: DataTypes.STRING,
      allowNull: true
    },
    rejectedBy: {
      type: DataTypes.STRING,
      allowNull: true
    }
  }, {
    sequelize,
    modelName: 'Complaint',
    paranoid: true
  })

  Complaint.associate = (models) => {
    Complaint.belongsTo(models.userDetail, {
      foreignKey: 'complaintById',
      targetKey: 'brgyKey'
    })

    Complaint.belongsTo(models.dataOption, {
      foreignKey: 'typeId',
      targetKey: 'brgyKey'
    })
  }

  return Complaint
}