'use strict'
const { Sequelize, DataTypes } = require('sequelize')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const db = {}

log.info('Environment: ' + process.env.NODE_ENV)

let sequelize
sequelize = new Sequelize(
  config.database,
  config.user,
  config.password,
  {
    host: config.host,
    dialect: config.dialect,
    operatorsAliases: 0,
    logging: false,
    pool: {
      max: config.pool.max,
      min: config.pool.min,
      acquire: config.pool.acquire,
      idle: config.pool.idle
    }
  }
)

sequelize.authenticate()
  .then(() => {
    log.info('Database Connected...')
  }).catch((error) => {
    log.fatal(error)
  })

db.sequelize = sequelize
db.Sequelize = Sequelize

db.roles = require('./roles.model.js')(sequelize, DataTypes)
db.users = require('./users.model.js')(sequelize, DataTypes)
db.dataOptionTypes = require('./dataOptionTypes.model.js')(sequelize, DataTypes)
db.dataOptions = require('./dataOptions.model.js')(sequelize, DataTypes)
db.countries = require('./countries.model.js')(sequelize, DataTypes)
db.regions = require('./regions.model.js')(sequelize, DataTypes)
db.cities = require('./cities.model.js')(sequelize, DataTypes)
db.barangays = require('./barangays.model.js')(sequelize, DataTypes)
db.userDetails = require('./userDetails.model.js')(sequelize, DataTypes)
db.complaints = require('./complaints.model.js')(sequelize, DataTypes)
db.banners = require('./banners.model.js')(sequelize, DataTypes)
db.announcements = require('./announcements.model.js')(sequelize, DataTypes)
db.news = require('./news.model.js')(sequelize, DataTypes)
db.teams = require('./team.model.js')(sequelize, DataTypes)
db.emergencies = require('./emergency.model.js')(sequelize, DataTypes)
db.businesses = require('./businesses.model.js')(sequelize, DataTypes)
db.employees = require('./employees.model.js')(sequelize, DataTypes)
db.requestDocument = require('./requestDocument.model.js')(sequelize, DataTypes)
db.registration = require('./registration.model.js')(sequelize, DataTypes)
db.dataLogs = require('./dataLogs.model.js')(sequelize, DataTypes)
db.inquiries = require('./Inquiry.model.js')(sequelize, DataTypes)
db.carStickers = require('./carSticker.model.js')(sequelize, DataTypes)

// Relations
// User Login
db.roles.hasMany(db.users, { as: 'role', foreignKey: 'roleId', targetKey: 'brgyKey' })
db.users.belongsTo(db.roles, { as: 'role', foreignKey: 'roleId', targetKey: 'brgyKey' })

db.dataOptions.hasMany(db.users, { as: 'status', foreignKey: 'statusId', targetKey: 'brgyKey' })
db.users.belongsTo(db.dataOptions, { as: 'status', foreignKey: 'statusId', targetKey: 'brgyKey' })

// Data Options
db.dataOptionTypes.hasMany(db.dataOptions, { as: 'dataOptionType', foreignKey: 'dataOptionTypeId', targetKey: 'brgyKey' })
db.dataOptions.belongsTo(db.dataOptionTypes, { as: 'dataOptionType', foreignKey: 'dataOptionTypeId', targetKey: 'brgyKey' })

// Regions
db.countries.hasMany(db.regions, { as: 'country', foreignKey: 'countryId', targetKey: 'brgyKey' })
db.regions.belongsTo(db.countries, { as: 'country', foreignKey: 'countryId', targetKey: 'brgyKey' })

// Cities
db.regions.hasMany(db.cities, { as: 'region', foreignKey: 'regionId', targetKey: 'brgyKey' })
db.cities.belongsTo(db.regions, { as: 'region', foreignKey: 'regionId', targetKey: 'brgyKey' })

// Brangays
db.cities.hasMany(db.barangays, { as: 'city', foreignKey: 'cityId', targetKey: 'brgyKey' })
db.barangays.belongsTo(db.cities, { as: 'city', foreignKey: 'cityId', targetKey: 'brgyKey' })

// UserDetails
db.dataOptions.hasMany(db.userDetails, { as: 'gender', foreignKey: 'genderId', targetKey: 'brgyKey' })
db.userDetails.belongsTo(db.dataOptions, { as: 'gender', foreignKey: 'genderId', targetKey: 'brgyKey' })

db.dataOptions.hasMany(db.userDetails, { as: 'street', foreignKey: 'streetId', targetKey: 'brgyKey' })
db.userDetails.belongsTo(db.dataOptions, { as: 'street', foreignKey: 'streetId', targetKey: 'brgyKey' })

db.dataOptions.hasMany(db.userDetails, { as: 'natureOfStay', foreignKey: 'natureOfStayId', targetKey: 'brgyKey' })
db.userDetails.belongsTo(db.dataOptions, { as: 'natureOfStay', foreignKey: 'natureOfStayId', targetKey: 'brgyKey' })

db.users.hasMany(db.userDetails, { as: 'user', foreignKey: 'loginId', targetKey: 'brgyKey' })
db.userDetails.belongsTo(db.users, { as: 'user', foreignKey: 'loginId', targetKey: 'brgyKey' })

db.barangays.hasMany(db.userDetails, { as: 'barangay', foreignKey: 'barangayId', targetKey: 'brgyKey' })
db.userDetails.belongsTo(db.barangays, { as: 'barangay', foreignKey: 'barangayId', targetKey: 'brgyKey' })

db.cities.hasMany(db.userDetails, { as: 'userCity', foreignKey: 'cityId', targetKey: 'brgyKey' })
db.userDetails.belongsTo(db.cities, { as: 'userCity', foreignKey: 'cityId', targetKey: 'brgyKey' })

db.regions.hasMany(db.userDetails, { as: 'userRegion', foreignKey: 'regionId', targetKey: 'brgyKey' })
db.userDetails.belongsTo(db.regions, { as: 'userRegion', foreignKey: 'regionId', targetKey: 'brgyKey' })

db.countries.hasMany(db.userDetails, { as: 'userCountry', foreignKey: 'countryId', targetKey: 'brgyKey' })
db.userDetails.belongsTo(db.countries, { as: 'userCountry', foreignKey: 'countryId', targetKey: 'brgyKey' })

// Complaints
db.dataOptions.hasMany(db.complaints, { as: 'type', foreignKey: 'typeId', targetKey: 'brgyKey' })
db.complaints.belongsTo(db.dataOptions, { as: 'type', foreignKey: 'typeId', targetKey: 'brgyKey' })

db.userDetails.hasMany(db.complaints, { as: 'complaintBy', foreignKey: 'complaintById', targetKey: 'brgyKey' })
db.complaints.belongsTo(db.userDetails, { as: 'complaintBy', foreignKey: 'complaintById', targetKey: 'brgyKey' })

// News
db.userDetails.hasMany(db.news, { as: 'createdBy', foreignKey: 'createdById', targetKey: 'brgyKey' })
db.news.belongsTo(db.userDetails, { as: 'createdBy', foreignKey: 'createdById', targetKey: 'brgyKey' })

// Emergency
db.userDetails.hasMany(db.emergencies, { as: 'emergencyBy', foreignKey: 'emergencyById', targetKey: 'brgyKey' })
db.emergencies.belongsTo(db.userDetails, { as: 'emergencyBy', foreignKey: 'emergencyById', targetKey: 'brgyKey' })

// Business
db.userDetails.hasMany(db.businesses, { as: 'owner', foreignKey: 'ownerId', targetKey: 'brgyKey' })
db.businesses.belongsTo(db.userDetails, { as: 'owner', foreignKey: 'ownerId', targetKey: 'brgyKey' })

// Employee
db.dataOptions.hasMany(db.employees, { as: 'position', foreignKey: 'positionId', targetKey: 'brgyKey'})
db.employees.belongsTo(db.dataOptions, { as: 'position', foreignKey: 'positionId', targetKey: 'brgyKey'})

// Request Document
db.userDetails.hasMany(db.requestDocument, { as: 'requestedBy', foreignKey: 'requestedById', targetKey: 'brgyKey' })
db.requestDocument.belongsTo(db.userDetails, { as: 'requestedBy', foreignKey: 'requestedById', targetKey: 'brgyKey' })

db.dataOptions.hasMany(db.requestDocument, { as: 'requestStatus', foreignKey: 'statusId', targetKey: 'brgyKey' })
db.requestDocument.belongsTo(db.dataOptions, { as: 'requestStatus', foreignKey: 'statusId', targetKey: 'brgyKey' })

db.dataOptions.hasMany(db.requestDocument, { as: 'civilStatus', foreignKey: 'civilStatusId', targetKey: 'brgyKey' })
db.requestDocument.belongsTo(db.dataOptions, { as: 'civilStatus', foreignKey: 'civilStatusId', targetKey: 'brgyKey' })

db.dataOptions.hasMany(db.requestDocument, { as: 'documentCategory', foreignKey: 'documentCategoryId', targetKey: 'brgyKey' })
db.requestDocument.belongsTo(db.dataOptions, { as: 'documentCategory', foreignKey: 'documentCategoryId', targetKey: 'brgyKey' })

db.dataOptions.hasMany(db.requestDocument, { as: 'purpose', foreignKey: 'purposeId', targetKey: 'brgyKey' })
db.requestDocument.belongsTo(db.dataOptions, { as: 'purpose', foreignKey: 'purposeId', targetKey: 'brgyKey' })

// CarSticker
db.userDetails.hasMany(db.carStickers, { as: 'carStickerBy', foreignKey: 'requestedById', targetKey: 'brgyKey' })
db.carStickers.belongsTo(db.userDetails, { as: 'carStickerBy', foreignKey: 'requestedById', targetKey: 'brgyKey' })

db.sequelize.sync({ force: false, alter: false }).then(() => log.info('Sync Completed...')).catch((error) => log.fatal('Sync Failed...', error))

module.exports = db
