const announcementController = require('../controllers/announcement.controller')
const { authenticateToken } = require('../services/authenticate.service')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const multer = require('multer')
const mb = 1000000
const router = require('express').Router()

const announcementStorage = multer.diskStorage({
 destination: function (req, file, cb) {
   cb(null, `${config.filePath}announcements`)
 },
 filename: function (req, file, cb) {
   if (file || file !== undefined) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9)
    cb(null, file.fieldname + '-' + uniqueSuffix  + '-' + file.originalname)
   } else {
    log.fatal('Error uploading image: No Attachment Found...')
    cb('ERROR: NO ATTACHMENT FOUND...', false)
   }
 },
 fileFilter: (req, file, cb) => {
  let valid = (file.mimetype === 'image/jpeg' || file.mimetype === 'image/jpg' || file.mimetype === 'image/png' || file.mimetype === 'image/gif')
  cb(null, valid)
 }
})

const uploadComplaint = multer({
 limits: { fileSize: 50 * mb },
 storage: announcementStorage
})

// Add
router.post('/', authenticateToken, announcementController.add)

// Add With Attachment
router.post('/with-attachment', authenticateToken, uploadComplaint.single('file'), (req, res) => {
  req.body.imagePath = `images/announcements/${req.file.filename}`
  return announcementController.addAndUpload(req, res)
})

// Add With Attachment
router.put('/with-attachment/:id', authenticateToken, uploadComplaint.single('file'), (req, res) => {
  req.body.imagePath = `images/announcements/${req.file.filename}`
  return announcementController.addAndUploadUpdate(req, res)
})

// Get All (For Admin)
router.get('/search', authenticateToken, announcementController.getAll)

// Get All (Not Disabled)
router.get('/', authenticateToken, announcementController.get)

// Get Filtered (Admin)
router.post('/filter', authenticateToken, announcementController.getFiltered)

// Get Filtered (Client)
router.post('/cfilter', announcementController.getFilteredClient)

// Get One
router.get('/:id', authenticateToken, announcementController.getOne)

// Update One
router.put('/:id', authenticateToken, announcementController.update)

// Delete
router.delete('/:id', authenticateToken, announcementController.remove)

module.exports = router