const bannerController = require('../controllers/banner.controller')
const { authenticateToken } = require('../services/authenticate.service')
const middleware = require('./middleware')
const router = require('express').Router()

// Middleware for uploading and optimizing banner images
const uploadBannerImage = middleware.uploadBanner();

// Add
router.post('/', authenticateToken, bannerController.add)

// Add With Attachment
router.post('/with-attachment', authenticateToken, uploadBannerImage, (req, res, next) => {
  if (!req.optimizedImagePath) {
      // If image optimization failed, return an error response
      return res.status(500).json({ error: 'Failed to upload and optimize image' });
  }
  // Set the image path in the request body
  req.body.imagePath = req.optimizedImagePath;
  // Call the controller method to add and upload
  return bannerController.addAndUpload(req, res, next);
});

// router.post('/with-attachment', authenticateToken, uploadComplaint.single('file'), (req, res) => {
//   req.body.imagePath = `images/banners/${req.file.filename}`
//   return bannerController.addAndUpload(req, res)
// })

// Add With Attachment
router.put('/with-attachment/:id', authenticateToken, uploadBannerImage, (req, res) => {
  if (!req.optimizedImagePath) {
      // If image optimization failed, return an error response
      return res.status(500).json({ error: 'Failed to upload and optimize image' });
  }
  // Set the image path in the request body
  req.body.imagePath = req.optimizedImagePath;
  // req.body.imagePath = `images/banners/${req.file.filename}`
  return bannerController.addAndUploadUpdate(req, res)
})

// Get All (For Admin)
router.get('/search', authenticateToken, bannerController.getAll)

// Get All (Not Disabled)
router.get('/', authenticateToken, bannerController.get)

// Get Filtered (Admin)
router.post('/filter', authenticateToken, bannerController.getFiltered)

// Get Filtered (Client)
router.post('/cfilter', bannerController.getFilteredClient)

// Get One
router.get('/:id', authenticateToken, bannerController.getOne)

// Update One
router.put('/:id', authenticateToken, bannerController.update)

// Delete
router.delete('/:id', authenticateToken, bannerController.remove)

module.exports = router