const businessController = require('../controllers/business.controller')
const { authenticateToken } = require('../services/authenticate.service')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const multer = require('multer')
const mb = 1000000
const router = require('express').Router()

const businessStorage = multer.diskStorage({
 destination: function (req, file, cb) {
   cb(null, `${config.filePath}business`)
 },
 filename: function (req, file, cb) {
   if (file || file !== undefined) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9)
    cb(null, file.fieldname + '-' + uniqueSuffix  + '-' + file.originalname)
   } else {
    log.fatal('Error uploading image: No Attachment Found...')
    cb('ERROR: NO ATTACHMENT FOUND...', false)
   }
 },
 fileFilter: (req, file, cb) => {
  let valid = (file.mimetype === 'image/jpeg' || file.mimetype === 'image/jpg' || file.mimetype === 'image/png' || file.mimetype === 'image/gif')
  cb(null, valid)
 }
})

const uploadBusiness = multer({ 
 limits: { fileSize: 50 * mb },
 storage: businessStorage
})

// Add
router.post('/', authenticateToken, businessController.add)

// Add With Attachment
router.post('/with-attachment', authenticateToken, uploadBusiness.array('file', 12), (req, res) => {
  let fileList = []
  req.files.forEach((file) => {
    fileList.push(`images/businesss/${file.filename}`)
  })
  req.body.imagePath = fileList.join(',')
  return businessController.addAndUpload(req, res)
})

// Get All (For Admin)
router.get('/search', authenticateToken, businessController.getAll)

// Get All (Not Disabled)
router.get('/', authenticateToken, businessController.get)

// Get Filtered (Admin)
router.post('/filter', authenticateToken, businessController.getFiltered)

// Get One
router.get('/:id', authenticateToken, businessController.getOne)

// Update One
router.put('/:id', authenticateToken, businessController.update)

// Delete
router.delete('/:id', authenticateToken, businessController.remove)

// Reject Business
router.post('/reject/:id', authenticateToken, businessController.rejectBusiness)

// Approve Business
router.post('/resolve/:id', authenticateToken, businessController.resolveBusiness)

module.exports = router