const carStickerController = require('../controllers/carSticker.controller')
const { authenticateToken } = require('../services/authenticate.service')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const multer = require('multer')
const mb = 1000000
const router = require('express').Router()

const carStickerStorage = multer.diskStorage({
 destination: function (req, file, cb) {
   cb(null, `${config.filePath}car-sticker`)
 },
 filename: function (req, file, cb) {
   if (file || file !== undefined) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9)
    cb(null, file.fieldname + '-' + uniqueSuffix  + '-' + file.originalname)
   } else {
    log.fatal('Error uploading image: No Attachment Found...')
    cb('ERROR: NO ATTACHMENT FOUND...', false)
   }
 },
 fileFilter: (req, file, cb) => {
  let valid = (file.mimetype === 'image/jpeg' || file.mimetype === 'image/jpg' || file.mimetype === 'image/png' || file.mimetype === 'image/gif')
  cb(null, valid)
 }
})

const uploadCarSticker = multer({ 
 limits: { fileSize: 50 * mb },
 storage: carStickerStorage
})

// Upload orcr / vehiclePhoto - Create Request Car Sticker
router.post('/request', authenticateToken, uploadCarSticker.fields([{ name: 'orcr', maxCount: 1 }, { name: 'vehiclePhoto', maxCount: 1 }]), (req, res) => {
  req.body.orcrPath = `images/car-sticker/${req.files.orcr[0].filename}`
  req.body.vehiclePhotoPath = `images/car-sticker/${req.files.vehiclePhoto[0].filename}`
  return carStickerController.createRequestCarSticker(req, res)
})

// Get All
router.get('/search', authenticateToken, carStickerController.getAll)

// Get All Enabled
router.get('/', authenticateToken, carStickerController.get)

// Get Filtered (Admin)
router.post('/filter', authenticateToken, carStickerController.getFiltered)

// Get One
router.get('/:id', authenticateToken, carStickerController.getOne)

// Update One
router.put('/:id', authenticateToken, carStickerController.update)

// Delete
router.delete('/:id', authenticateToken, carStickerController.remove)

module.exports = router