const newsController = require('../controllers/news.controller')
const { authenticateToken } = require('../services/authenticate.service')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const multer = require('multer')
const mb = 1000000
const router = require('express').Router()

const newstorage = multer.diskStorage({
 destination: function (req, file, cb) {
   cb(null, `${config.filePath}news`)
 },
 filename: function (req, file, cb) {
   if (file || file !== undefined) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9)
    cb(null, file.fieldname + '-' + uniqueSuffix  + '-' + file.originalname)
   } else {
    log.fatal('Error uploading image: No Attachment Found...')
    cb('ERROR: NO ATTACHMENT FOUND...', false)
   }
 },
 fileFilter: (req, file, cb) => {
  let valid = (file.mimetype === 'image/jpeg' || file.mimetype === 'image/jpg' || file.mimetype === 'image/png' || file.mimetype === 'image/gif')
  cb(null, valid)
 }
})

const uploadComplaint = multer({
 limits: { fileSize: 50 * mb },
 storage: newstorage
})

// Add
router.post('/', authenticateToken, newsController.add)

// Add With Attachment
router.post('/with-attachment', authenticateToken, uploadComplaint.single('file'), (req, res) => {
  req.body.imagePath = `images/news/${req.file.filename}`
  return newsController.addAndUpload(req, res)
})

// Add With Attachment
router.put('/with-attachment/:id', authenticateToken, uploadComplaint.single('file'), (req, res) => {
  req.body.imagePath = `images/news/${req.file.filename}`
  return newsController.addAndUploadUpdate(req, res)
})

// Get All (For Admin)
router.get('/search', authenticateToken, newsController.getAll)

// Get All (Not Disabled)
router.get('/', authenticateToken, newsController.get)

// Get Filtered (Admin)
router.post('/filter', authenticateToken, newsController.getFiltered)

// Get Filtered (Client)
router.post('/cfilter', newsController.getFilteredClient)

// Get One
router.get('/:id', authenticateToken, newsController.getOne)

// Update One
router.put('/:id', authenticateToken, newsController.update)

// Delete
router.delete('/:id', authenticateToken, newsController.remove)

module.exports = router