const authenticationController = require('../controllers/login.controller')
const dataOptionController = require('../controllers/dataOption.controller')
const dataOptionTypeController = require('../controllers/dataOptionType.controller')
const feController = require('../controllers/fe.controller')
const userController = require('../controllers/user.controller')
const userDetailController = require('../controllers/userDetail.controller')
const announcementController = require('../controllers/announcement.controller')
const inquiryController = require('../controllers/inquiry.controller')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const multer = require('multer')
const mb = 1000000
const router = require('express').Router()


const verificationStorage = multer.diskStorage({
 destination: function (req, file, cb) {
   cb(null, `${config.filePath}verifications`)
 },
 filename: function (req, file, cb) {
   if (file || file !== undefined) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9)
    cb(null, file.fieldname + '-' + uniqueSuffix  + '-' + file.originalname)
   } else {
    log.fatal('Error uploading image: No Attachment Found...')
    cb('ERROR: NO ATTACHMENT FOUND...', false)
   }
 },
 fileFilter: (req, file, cb) => {
   let valid = (file.mimetype === 'image/jpeg' || file.mimetype === 'image/jpg' || file.mimetype === 'image/png' || file.mimetype === 'image/gif')
   cb(null, valid)
 }
})

const uploadVerification = multer({ 
 limits: { fileSize: 50 * mb },
 storage: verificationStorage
})

// POST Login (New Credential)
router.post('/login', authenticationController.login)

// GET Data Option
router.get('/data-option', dataOptionController.get)

// GET Data Option Type
router.get('/data-option-type', dataOptionTypeController.get)

// POST Add User Login Credentials
router.post('/user', userController.add)

// POST Register User Details
router.post('/register', userDetailController.add)

// INITIALIZE - Upon App Mounted
router.post('/initialize', feController.feInitialize)

router.post('/announcements', announcementController.getFilteredClient)

// Re-evaluate Token
router.post('/check/:token', feController.evaluateToken)

// Re-evaluate Refresh Token
router.post('/check-refresh/:token', feController.evaluateRefreshToken)

// Upload Verification (Unauthenticated)
router.post('/verification', uploadVerification.single('file'), (req, res) => {
  req.body.verificationPath = `images/verifications/${req.file.filename}`
  return userDetailController.uploadVerification(req, res)
})

// Public Contact Us
router.post('/inquire', inquiryController.addNewInquiry)

// POST Upload Identification Image

module.exports = router