const requestDocumentController = require('../controllers/requestDocument.controller')
const { authenticateToken } = require('../services/authenticate.service')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const multer = require('multer')
const mb = 1000000
const router = require('express').Router()

const requestDocumentStorage = multer.diskStorage({
 destination: function (req, file, cb) {
   cb(null, `${config.filePath}request-document`)
 },
 filename: function (req, file, cb) {
   if (file || file !== undefined) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9)
    cb(null, file.fieldname + '-' + uniqueSuffix  + '-' + file.originalname)
   } else {
    log.fatal('Error uploading image: No Attachment Found...')
    cb('ERROR: NO ATTACHMENT FOUND...', false)
   }
 },
 fileFilter: (req, file, cb) => {
  let valid = (file.mimetype === 'image/jpeg' || file.mimetype === 'image/jpg' || file.mimetype === 'image/png' || file.mimetype === 'image/gif')
  cb(null, valid)
 }
})

const uploadRequestDocument = multer({ 
 limits: { fileSize: 50 * mb },
 storage: requestDocumentStorage
})

// Upload UserPhoto / CedulaPhoto - Create Request Document
router.post('/certificate', authenticateToken, uploadRequestDocument.fields([{ name: 'user', maxCount: 1 }, { name: 'cedula', maxCount: 1 }]), (req, res) => {
  req.body.userPath = `images/request-document/${req.files.user[0].filename}`
  req.body.cedulaPath = `images/request-document/${req.files.cedula[0].filename}`
  return requestDocumentController.createRequestDocumentWithCedula(req, res)
})

// Upload UserPhoto - Create Request Document / Certificate and Cedula
router.post('/certificate-cedula', authenticateToken, uploadRequestDocument.single('user'), (req, res) => {
  req.body.userPath = `images/request-document/${req.file.filename}`
  return requestDocumentController.createRequestDocumentWithoutCedula(req, res)
})

// Create Request Document / Cedula
router.post('/cedula', authenticateToken, (req, res) => {
  return requestDocumentController.createRequestDocumentCedula(req, res)
})

// Get All
router.get('/search', authenticateToken, requestDocumentController.getAll)

// Get All Enabled
router.get('/', authenticateToken, requestDocumentController.get)

// Get Filtered (Admin)
router.post('/filter', authenticateToken, requestDocumentController.getFiltered)

// Get One
router.get('/:id', authenticateToken, requestDocumentController.getOne)

// Update One
router.put('/:id', authenticateToken, requestDocumentController.update)

// Delete
router.delete('/:id', authenticateToken, requestDocumentController.remove)

module.exports = router