const userDetailController = require('../controllers/userDetail.controller')
const { authenticateToken } = require('../services/authenticate.service')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()
const multer = require('multer')
const router = require('express').Router()
const mb = 1000000

const profileStorage = multer.diskStorage({
 destination: function (req, file, cb) {
   cb(null, `${config.filePath}profiles`)
 },
 filename: function (req, file, cb) {
   if (file || file !== undefined) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9)
    cb(null, file.fieldname + '-' + uniqueSuffix  + '-' + file.originalname)
   } else {
    log.fatal('Error uploading image: No Attachment Found...')
    cb('ERROR: NO ATTACHMENT FOUND...', false)
   }
 },
 fileFilter: (req, file, cb) => {
  let valid = (file.mimetype === 'image/jpeg' || file.mimetype === 'image/jpg' || file.mimetype === 'image/png' || file.mimetype === 'image/gif')
  cb(null, valid)
 }
})
const complaintStorage = multer.diskStorage({
 destination: function (req, file, cb) {
   cb(null, `${config.filePath}complaints`)
 },
 filename: function (req, file, cb) {
   if (file || file !== undefined) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9)
    cb(null, file.fieldname + '-' + uniqueSuffix  + '-' + file.originalname)
   } else {
    log.fatal('Error uploading image: No Attachment Found...')
    cb('ERROR: NO ATTACHMENT FOUND...', false)
   }
 },
 fileFilter: (req, file, cb) => {
  let valid = (file.mimetype === 'image/jpeg' || file.mimetype === 'image/jpg' || file.mimetype === 'image/png' || file.mimetype === 'image/gif')
  cb(null, valid)
 }
})
const verificationStorage = multer.diskStorage({
 destination: function (req, file, cb) {
   cb(null, `${config.filePath}verifications`)
 },
 filename: function (req, file, cb) {
   if (file || file !== undefined) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9)
    cb(null, file.fieldname + '-' + uniqueSuffix  + '-' + file.originalname)
   } else {
    log.fatal('Error uploading image: No Attachment Found...')
    cb('ERROR: NO ATTACHMENT FOUND...', false)
   }
 },
 fileFilter: (req, file, cb) => {
   let valid = (file.mimetype === 'image/jpeg' || file.mimetype === 'image/jpg' || file.mimetype === 'image/png' || file.mimetype === 'image/gif')
   cb(null, valid)
 }
})

const uploadProfile = multer({ 
 limits: { fileSize: 50 * mb },
 storage: profileStorage
})
const uploadComplaint = multer({ 
 limits: { fileSize: 50 * mb },
 storage: complaintStorage
})
const uploadVerification = multer({ 
 limits: { fileSize: 50 * mb },
 storage: verificationStorage
})

// Upload Verification (Authenticated)
router.post('/verification', authenticateToken, uploadVerification.single('file'), async (req, res) => {
  // Update User Details Verification
  req.body.verificationPath = `images/verifications/${req.file.filename}`
  return userDetailController.uploadVerification(req, res)
})

// Upload Profile (Authenticated)
router.post('/profile', authenticateToken, uploadProfile.single('file'), (req, res) => {
 // Update User Details Photo
 req.body.imagePath = `images/profiles/${req.file.filename}`
 return userDetailController.uploadPhoto(req, res)
})

// Upload Complain (Authenticated)
router.post('/complaint', authenticateToken, uploadComplaint.array('file', 12), (req, res) => {
 // if (!req.file || req.file === undefined) {
 //  return res.status(400).send({ statusCode: 400, message: 'No Images Selected' })
 // } else {
 //  return res.status(200).send({ statusCode: 200, message: 'Image Uploaded', file: `public/images/${req.file.filename}` })
 // }
})

module.exports = router