const userController = require('../controllers/user.controller')
const { authenticateToken } = require('../services/authenticate.service')
const router = require('express').Router()

// Add
router.post('/', authenticateToken, userController.add)

// Get All (For Admin)
router.get('/search', authenticateToken, userController.getAll)

// Get All (Not Disabled)
router.get('/', authenticateToken, userController.get)

// Get User By Email
router.get('/email', async (req, res) => {
  const result = await userController.findUserByEmail(req, res)
  if (!result) {
    return res.status(200).send({
     statusCode: 200,
     message: 'Email Address Available.',
     content:result
    })
  } else {
    return res.status(400).send({
     statusCode: 400,
     message: 'Email Address Not Available.'
    })
  }
})

// Reset Login Attempts
router.post('/reset-login-attempts/:id', userController.resetLoginAttempts)

// Disabled Account
router.post('/disabled/:id', userController.disabledAccount)

// Reject Account
router.post('/reject/:id', userController.rejectedAccount)

// Approve Account
router.post('/approve/:id', userController.approveAccount)

// Get Filtered (Admin)
router.post('/filter', authenticateToken, userController.getFiltered)

// Get One
router.get('/:id', authenticateToken, userController.getOne)

// Update One
router.put('/:id', authenticateToken, userController.update)

// Delete
router.delete('/:id', authenticateToken, userController.remove)

module.exports = router