require('dotenv').config()
const config = require('./config')[process.env.NODE_ENV || 'development']
const log = config.log()
const cors = require('cors')
const express = require('express')
const bodyParser = require('body-parser')
const history = require('connect-history-api-fallback')
const helmet = require('helmet')
const cookieParser = require('cookie-parser')
const path = require('path')
const compression = require('compression')
const passport = require('passport')
const { doLogout } = require('./services/authenticate.service')

var corsOptions = {
 origin: ['http://localhost:8080', 'https://1namayan.com', 'https://www.1namayan.com', 'https://app.1namayan.com', 'https://test18.1namayan.com']
}

require('./config/passport')(passport)

const app = express()

app.use(helmet.hsts({
 /* ONE_YEAR */
 maxAge: 31536000000,
 includeSubDomains: true,
 force: true
}))

app.use(helmet.frameguard())
app.use(helmet.xssFilter())
app.use(helmet.noSniff());
app.use(cookieParser())

app.use(express.static(path.join(__dirname, '.')))

// Middleware
app.use(bodyParser.json())
app.use(bodyParser.urlencoded({ extended: true }))
app.use(passport.initialize())
app.use(cors(corsOptions))
app.use(compression())
app.use(history({
 disableDotRule: true,
 verbose: true
}));

// Routers
const feRouter = require('./routes/fe.routes.js')
const publicRouter = require('./routes/public.router.js')
const initRouter = require('./routes/initialize.router.js')
const roleRouter = require('./routes/role.router.js')
const userRouter = require('./routes/user.router.js')
const barangayRouter = require('./routes/barangay.router.js')
const cityRouter = require('./routes/city.router.js')
const complaintRouter = require('./routes/complaint.router.js')
const countryRouter = require('./routes/country.router.js')
const dataOptionRouter = require('./routes/dataOption.router.js')
const dataOptionTypeRouter = require('./routes/dataOptionType.router.js')
const regionRouter = require('./routes/region.router.js')
const userDetailRouter = require('./routes/userDetail.router.js')
const loginRouter = require('./routes/login.router.js')
const authRouter = require('./routes/auth.router.js')
const resetPasswordRouter = require('./routes/resetPassword.router.js')
const uploadRouter = require('./routes/upload.router.js')
const bannerRouter = require('./routes/banner.router.js')
const announcementRouter = require('./routes/announcement.router.js')
const newsRouter = require('./routes/news.router.js')
const team = require('./routes/team.router.js')
const emergency = require('./routes/emergency.router.js')
const requestDocument = require('./routes/requestDocument.router.js')
const dataLogs = require('./routes/DataLogs.router.js')
const sms = require('./routes/Sms.router.js')
const carSticker = require('./routes/carSticker.router.js')

// New Routes
const register = require('./routes/registration.router.js')

app.use('/api/fe', feRouter)
app.use('/api/public', publicRouter)
app.use('/api/initialize', initRouter)
app.use('/api/role', roleRouter)
app.use('/api/user', userRouter)
app.use('/api/barangay', barangayRouter)
app.use('/api/city', cityRouter)
app.use('/api/complaint', complaintRouter)
app.use('/api/country', countryRouter)
app.use('/api/data-option', dataOptionRouter)
app.use('/api/data-option-type', dataOptionTypeRouter)
app.use('/api/region', regionRouter)
app.use('/api/user-detail', userDetailRouter)
app.use('/api/login', loginRouter)
app.use('/api/auth', authRouter)
app.use('/api/reset-password', resetPasswordRouter)
app.use('/api/upload', uploadRouter)
app.use('/api/banner', bannerRouter)
app.use('/api/announcement', announcementRouter)
app.use('/api/news', newsRouter)
app.use('/api/team', team)
app.use('/api/car-sticker', carSticker)
app.use('/api/emergency', emergency)
app.use('/api/request-document', requestDocument)
app.use('/api/register', register)
app.use('/api/logs', dataLogs)
app.use('/api/sms', sms)

// Test API
app.get('/api/test', (req, res) => {
 res.json({ message: 'Successfuly Connect to Server.' })
})

app.post('/api/logout', (req, res) => {
  return doLogout(req, res)
})

// Port
try {
  if (process.env.NODE_ENV === 'production') {
    const port = process.env.PORT || 3000
    app.listen(port, () => log.info(`Production Server started on port ${port}`))
  } else if (process.env.NODE_ENV === 'development') {
    const port = process.env.PORT || 3000
    app.listen(port, () => log.info(`Development Server started on port ${port}`))
  } else {
    const port = process.env.PORT || 5000
    app.listen(port, () => log.info(`Development Server started on port ${port}`))
  }
} catch (error) {
  log.fatal(error)
}