const bcrypt = require('bcrypt')
const userController = require('../controllers/user.controller')
const config = require('../config')[process.env.NODE_ENV || 'development']
const log = config.log()

// LOGIN BUSINESS RULES
// 1. IF EMAIL DOES'T EXIST - RETURN
// 2. IF PASSWORD DO NOT MATCH - ADD ATTEMPTS / RETURN
// 3. IF LOGIN ATTEMPT REACHES 10 - ACCOUNT DISABLED / RETURN
// 4. IF LOGGED IN SUCCESSFULLY - RESET ATTEMPTS / RETURN

async function loginRules(req, res) {
 try {
   const request = req.body
   const result = await userController.findUserByEmailFn(req, res)
   // Reject if there is no result found
   if (!result) return { statusCode: 200, code: 'EMPTY', message: `Account Doesn't Exist` }

   // If Account is Disabled / Deleted - Redirect to disabled account
   if (result.disabled || result.deleted) return { statusCode: 200, code: 'DISABLED', message: 'Account Disabled' }
   // Compare Password
   return await bcrypt.compare(request.password, result.password)
   .then((trueOrFalse) => {
     return processLogin(req, res, trueOrFalse, result)
   }).catch((error) => {
    log.fatal(error)
    return { statusCode: 400, code: 'ERROR', message: 'Invalid Email / Password' }
   })
  } catch (error) {
   return { statusCode: 400, code: 'ERROR', message: error }
  }
}

async function processLogin(req, res, trueOrFalse, result) {
 try {
  req.body = { ...result }
  if (trueOrFalse) {
    const response = await userController.resetLoginAttemptsFn(req, res)
    if (response) return { statusCode: 200, code: 'SUCCESS', message: 'Success Login', content:result }
  } else {
   // If Login attempt reaches 10, disabled account
    let attemptCount = result.loginAttempts !== null ? result.loginAttempts + 1 : 1
    req.body.count = attemptCount

    if (attemptCount > 9) {
     const response = await userController.disabledAccountFn(req, res)
     if (response) return { statusCode: 200, code: 'DISABLED', message: 'Account Disabled' }
    } else {
     // Invalid Username / Password
     const response = await userController.addLoginAttemptsFn(req, res)
     if (response) return { statusCode: 200, code: 'INVALID', message: 'Invalid Email / Password' }
    }
  }
 } catch (error) {
   return { statusCode: 400, code: 'ERROR', message: error }
 }
}

module.exports = {
 loginRules
}